/*
 * $Id: wvcncm.c 1.54 1997/03/28 19:46:27 dumoulin Exp $
 */

/*-- This is the first line of WVCNCM.C -------------------------------*/

#include <windows.h>
#include <windowsx.h>
#include "wvglob.h"
#include "winvn.h"
#include "defaults.h"
#pragma hdrstop
#include <stdlib.h>
#include <limits.h>
#include <io.h>                                 // for _access
#include <ctype.h>                      // for isspace
#include <commctrl.h>
#define MAX_DIR_ENTRIES 8

void enableDemandLogonCtl (HWND hDlg, int forceType);
int SetupEncodingOptions (HWND hDlg, char *startEncoding, BOOL includeCustom, BOOL showNone);

BOOL ProcessEncodingOptions (HWND hDlg, char *newEncodingType);

/////////////////////////////////////////////////////////////////////////////
//   FUNCTION: CenterWindow (HWND, HWND)
//   PURPOSE:  Center one window over another
//   COMMENTS:
//      Dialog boxes take on the screen position that they were designed
//      at, which is not always appropriate. Centering the dialog over a
//      particular window usually results in a better position.
//
BOOL WINAPI CenterWindow (HWND hwndChild, HWND hwndParent)
{
	RECT    rChild, rParent= {0,0,0,0};
	int     wChild, hChild, wParent, hParent;
	int     wScreen, hScreen, xNew, yNew;
	HDC     hdc;
	WINDOWPLACEMENT wndpl;

	if (hwndParent == NULL)
		hwndParent = GetActiveWindow();
	if (hwndParent == NULL)
		hwndParent = GetDesktopWindow();

	wndpl.length = sizeof(WINDOWPLACEMENT);
	if (GetWindowPlacement( hwndParent, &wndpl ))
	{
		CopyRect(&rParent, &wndpl.rcNormalPosition);
	}
   
	// Get the Height and Width of the child window
	GetWindowRect (hwndChild, &rChild);
	wChild = rChild.right - rChild.left;
	hChild = rChild.bottom - rChild.top;

	// Get the Height and Width of the parent window
	//GetWindowRect (hwndParent, &rParent);
	wParent = rParent.right - rParent.left;
	hParent = rParent.bottom - rParent.top;

	// Get the display limits
	hdc = GetDC (hwndChild);
	wScreen = GetDeviceCaps (hdc, HORZRES);
	hScreen = GetDeviceCaps (hdc, VERTRES);
	ReleaseDC (hwndChild, hdc);

	// Calculate new X position, then adjust for screen
	xNew = rParent.left + ((wParent - wChild) /2);
	if (xNew < 0) 
	{
		xNew = 0;
	}
	else if ((xNew+wChild) > wScreen) 
	{
		xNew = wScreen - wChild;
	}

	// Calculate new Y position, then adjust for screen
	yNew = rParent.top  + ((hParent - hChild) /2);
	if (yNew < 0) 
	{
		yNew = 0;
	}
	else if ((yNew+hChild) > hScreen) 
	{
		yNew = hScreen - hChild;
	}

	// Set it, and return
	return SetWindowPos (hwndChild, NULL,
			xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}



/*-- function WinVnCommDlg ---------------------------------------------
 *
 *   Dialog function to process the Configure Communications
 *   dialog box.
 */

BOOL CALLBACK WinVnCommDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
#define MAXSTR  32
	static int SaveDemand, saveMailForceType;
	static unsigned int saveNewsForceType;

	int mail_force_radio, news_force_radio;
	char NNTPPassword[MAXNNTPSIZE];
	BOOL bOK;

	static HFONT hDlgFont = NULL;
	

	switch (iMessage) 
	{

	case WM_INITDIALOG:
		CenterWindow (hDlg, GetParent(hDlg) );
		/* Get dialog font and create non-bold version */ 
		hDlgFont = NULL;
		if (g_bIsXP && !hDlgFont)
		{  //fix password font
			LOGFONT lFont;
			lFont.lfHeight = 14;
			lFont.lfWidth = 0;
			lFont.lfEscapement = 0;
			lFont.lfOrientation = 0;
			lFont.lfWeight = 200; // non-bold font weight
			lFont.lfItalic = 0;
			lFont.lfUnderline = 0;
			lFont.lfStrikeOut = 0;
			lFont.lfCharSet = ANSI_CHARSET;
			lFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
			lFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
			lFont.lfQuality = DEFAULT_QUALITY;
			lFont.lfPitchAndFamily = VARIABLE_PITCH | FF_SWISS;
			strcpy(lFont.lfFaceName, "Microsoft Sans Serif");
			hDlgFont = CreateFontIndirect(&lFont);

			SendDlgItemMessage(hDlg, ID_CONFIG_AUTH_PASSWORD, WM_SETFONT,  (WPARAM)hDlgFont, (DWORD)TRUE);
			// Send WM_SETFONT message to desired controls

		}

		SetDlgItemText (hDlg, ID_CONFIG_NNTP_SERVER, NNTPHost);
		CheckDlgButton (hDlg, ID_CONFIG_CONNECT_AT_START, ConnectAtStartup);
#ifdef _INCLSSL
		CheckDlgButton (hDlg, ID_CONFIG_USESSL, UseSSL);
#else
		CheckDlgButton (hDlg, ID_CONFIG_USESSL, 0);
		SendDlgItemMessage (hDlg, ID_CONFIG_USESSL, WM_ENABLE, 0, 0);
#endif
		SetDlgItemText (hDlg, ID_CONFIG_SMTP_SERVER, SMTPHost);
		SetDlgItemText (hDlg, ID_CONFIG_NNTP_PORT, NNTPService);
		SetDlgItemText (hDlg, ID_CONFIG_AUTH_USERNAME, NNTPUserName);
		MRRDecrypt (NNTPPasswordEncrypted, (unsigned char *) NNTPPassword, MAXNNTPSIZE);
		SetDlgItemText (hDlg, ID_CONFIG_AUTH_PASSWORD, NNTPPassword);
		SetDlgItemText (hDlg, ID_CONFIG_MAPI_PREFIX, MAPIPrefix);
		SetDlgItemInt  (hDlg, ID_CONFIG_TIMEOUT, StallTimeout, FALSE);
		
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_NNTP_SERVER), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_NNTP_PORT), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_SMTP_SERVER), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_AUTH_USERNAME), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_AUTH_PASSWORD), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_MAPI_PREFIX), MAXPREFIXSIZE);

		saveMailForceType = MailForceType;
		if (MailForceType == -1)
			mail_force_radio = IDD_MAIL_SELECT_AUTO;
		else
			mail_force_radio = MailForceType + IDD_MAIL_SELECT_NONE;

		CheckRadioButton (hDlg, IDD_MAIL_SELECT_NONE,
								IDD_MAIL_SELECT_AUTO, mail_force_radio);

		saveNewsForceType = force_xhdr;
		if (force_xhdr == HDR_AUTO)
			news_force_radio = IDD_NEWS_SELECT_AUTO;
		else if (force_xhdr == HDR_XOVER)
			news_force_radio = IDD_NEWS_SELECT_XOVER;   
		else if (force_xhdr == HDR_XHDR)
			news_force_radio = IDD_NEWS_SELECT_XHDR;
		else
			news_force_radio = IDD_NEWS_SELECT_AUTO;

		CheckRadioButton (hDlg, IDD_NEWS_SELECT_BASE,
					  IDD_NEWS_SELECT_AUTO, news_force_radio);

		// PCT, 10/05/94
		CheckDlgButton (hDlg, IDD_SAVEPASSWORD, NNTPSavePassword);

		SaveDemand = MailDemandLogon;
		CheckDlgButton (hDlg, IDD_DEMANDLOGON, SaveDemand);
		enableDemandLogonCtl (hDlg, saveMailForceType);
		EnableWindow (GetDlgItem (hDlg, ID_CONFIG_SMTP_SERVER), (mail_force_radio == IDD_MAIL_SELECT_SMTP || mail_force_radio == IDD_MAIL_SELECT_AUTO));
		EnableWindow (GetDlgItem (hDlg, ID_CONFIG_MAPI_PREFIX), (mail_force_radio == IDD_MAIL_SELECT_MAPI || mail_force_radio == IDD_MAIL_SELECT_AUTO));
		if (lParam)
		{
			SetFocus( GetDlgItem (hDlg, ID_CONFIG_AUTH_PASSWORD));
			return FALSE;

		}
		return TRUE;
		break;

		case WM_HELP:
			{
				LPHELPINFO lphi = (LPHELPINFO) lParam; 
				
				DoHelp ((HWND)lphi->hItemHandle/*NetDoc.hDocWnd*/, HELP_CONTEXT, (DWORD)401);

				if (lphi->iContextType == HELPINFO_WINDOW && lphi->iCtrlId != 9 )
				{
					DWORD dwHelp[4] = {	lphi->iCtrlId, lphi->iCtrlId + 0x10000, 0,0};
					//dwHelp[0] = 401;
					//dwHelp[1] = 401 + 0x10000;
					//WinHelp ((HWND)lphi->hItemHandle/*NetDoc.hDocWnd*/, mybuf, HELP_WM_HELP, (DWORD)(LPVOID)dwHelp);
				}

			}
			break;

		case WM_COMMAND:
		switch (wParam) 
		{
			case IDHELP:
				DoHelp (hDlg, HELP_CONTEXT, (DWORD)401);
				break;

			case IDOK:
				ConnectAtStartup = (IsDlgButtonChecked (hDlg, ID_CONFIG_CONNECT_AT_START) != 0);

#ifdef _INCLSSL
				UseSSL = (IsDlgButtonChecked (hDlg, ID_CONFIG_USESSL) != 0);
#else
				UseSSL = 0;
#endif				
				GetDlgItemText (hDlg, ID_CONFIG_NNTP_SERVER, NNTPHost, MAXNNTPSIZE);
				if (*NNTPHost == '\0') 
				{
					WVMessageBox (hDlg, IDS_WVCNCM_ENTER_ADDR, IDS_WVCNCM_INVALID_SERVER, MB_OK | MB_ICONSTOP);
					break;
				}
				GetDlgItemText (hDlg, ID_CONFIG_NNTP_PORT, NNTPService, MAXNNTPSIZE);
				if (*NNTPService == '\0') 
				{
					WVMessageBox (hDlg, IDS_WVCNCM_ENTER_PORT, IDS_WVCNCM_INVALID_PORT, MB_OK | MB_ICONSTOP);
					break;
				}
				GetDlgItemText (hDlg, ID_CONFIG_SMTP_SERVER, SMTPHost, MAXNNTPSIZE);
				if (*SMTPHost == '\0' && saveMailForceType == MT_SMTP) 
				{
					WVMessageBox (hDlg, IDS_WVCNCM_INVALID_MAIL, IDS_WVCNCM_INVALID_SMTP, MB_OK | MB_ICONSTOP);
					saveMailForceType = MT_NONE;
				}
      
				GetDlgItemText (hDlg, ID_CONFIG_MAPI_PREFIX, MAPIPrefix, MAXPREFIXSIZE);

				// PCT, 10/05/94
				NNTPSavePassword = IsDlgButtonChecked (hDlg, IDD_SAVEPASSWORD);
				GetDlgItemText (hDlg, ID_CONFIG_AUTH_USERNAME, NNTPUserName, MAXNNTPSIZE);
				GetDlgItemText (hDlg, ID_CONFIG_AUTH_PASSWORD, NNTPPassword, MAXNNTPSIZE);
				{
					unsigned int Value = GetDlgItemInt(hDlg, ID_CONFIG_TIMEOUT, &bOK, FALSE);
					if (bOK && Value != StallTimeout)
					{
						StallTimeout = Value;
						InternalTimeoutCounter = StallTimeout * 1000 / COMM_TIMER_INTERVAL;
					}
				}
				MRREncrypt ((unsigned char *) NNTPPassword, strlen (NNTPPassword) + 1,
								  NNTPPasswordEncrypted);
				MailDemandLogon = SaveDemand;

				if ((saveMailForceType != MailForceType) || (saveNewsForceType != force_xhdr)) 
				{
					   /* change of mail transport medium
						  Not entirely correct as may change from
						  explicit selection to auto detect which
						  in this case will (but in reality may not)
						  affect a relogin to mail)  */
					MailForceType = saveMailForceType;
					force_xhdr = saveNewsForceType;
					if (force_xhdr == HDR_AUTO || force_xhdr == HDR_XOVER) 
					{
						xoverp = 1;
					}
					MailCrashExit (hDlg);
					MailInit (hDlg);
					UpdateAllMailMenus ();
				}
				EndDialog (hDlg, TRUE);
			break;

			case IDCANCEL:
				EndDialog (hDlg, FALSE);
				break;
			case ID_CONFIG_USESSL:
				if (IsDlgButtonChecked (hDlg, ID_CONFIG_USESSL) == BST_CHECKED)
				{
					SetDlgItemText (hDlg, ID_CONFIG_NNTP_PORT, "563");
				}
				else
				{
					SetDlgItemText (hDlg, ID_CONFIG_NNTP_PORT, "nntp");
				}
				break; 


			case IDD_MAIL_SELECT_NONE:
			case IDD_MAIL_SELECT_MAPI:        
			case IDD_MAIL_SELECT_SMTP:
				saveMailForceType = wParam - IDD_MAIL_SELECT_NONE;
				enableDemandLogonCtl (hDlg, saveMailForceType);
				EnableWindow (GetDlgItem (hDlg, ID_CONFIG_MAPI_PREFIX), (saveMailForceType == MT_MAPI));
				EnableWindow (GetDlgItem (hDlg, ID_CONFIG_SMTP_SERVER), (saveMailForceType == MT_SMTP));
				break;

			case IDD_MAIL_SELECT_AUTO:
				saveMailForceType = -1;
				enableDemandLogonCtl (hDlg, saveMailForceType);
				EnableWindow (GetDlgItem (hDlg, ID_CONFIG_SMTP_SERVER), ENABLE);
				EnableWindow (GetDlgItem (hDlg, ID_CONFIG_MAPI_PREFIX), ENABLE);
				break;

			case IDD_NEWS_SELECT_XOVER:
				saveNewsForceType = HDR_XOVER;
				break;

			case IDD_NEWS_SELECT_AUTO:
				saveNewsForceType = HDR_AUTO;
				break;

			case IDD_NEWS_SELECT_XHDR:
				saveNewsForceType = HDR_XHDR;
				break;

			case IDD_DEMANDLOGON:
				SaveDemand = !SaveDemand;
				CheckDlgButton (hDlg, IDD_DEMANDLOGON, SaveDemand);
				break; 
			case ID_CONFIG_ADVSMTP:
			      if (DialogBox (hInst, MAKEINTRESOURCE(IDD_WINVNSMTP), hDlg, lpfnWinVnSMTPDlg) == IDOK)
				  {
					SetDlgItemText (hDlg, ID_CONFIG_SMTP_SERVER, SMTPHost);
				  }
				  break;


			default:
				return FALSE;
		}
		break;

	default:
			return FALSE;
			break;
	}
	return TRUE;
}
/*
 *  enable/disable the check box for the DemandLogon variable
 *  as requested
 *
 *  some mail transports do not require authentification (logon)
 *  so demand (deferred ) logon is not an option 
 *
 */
void enableDemandLogonCtl (HWND hDlg, int forceType)
{
	EnableWindow (GetDlgItem (hDlg, IDD_DEMANDLOGON),
                  (forceType == -1) || (forceType == MT_MAPI));
}

/*-- function WinVnSaveArtDlg ---------------------------------------
 *
 *  Dialog function to handle Save Article dialog box.
 */

BOOL CALLBACK WinVnSaveArtDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  static int MyAppend;
  char fileName[MAXFILENAME];

  switch (iMessage) {

  case WM_INITDIALOG:
        {
          int i;
          char szEntry[20];
          HWND hList = GetDlgItem(hDlg, IDD_FILENAME);

          MyAppend = SaveArtAppend;
          CheckDlgButton (hDlg, IDD_APPEND, MyAppend);
          for (i = MAX_DIR_ENTRIES; i > 0; i--)
          {
              wsprintf(szEntry, "file%02d", i);
              GetPrivateProfileString (MRUENTRIES, szEntry, "", fileName, MAXFILENAME, szAppProFile);
              if (*fileName)
                SendMessage (hList, CB_INSERTSTRING, 0, (LONG) (LPSTR) fileName);

          }
          if (*SaveArtFileName)
            SetDlgItemText (hDlg, IDD_FILENAME, SaveArtFileName);
          else
            SendMessage(hList, CB_SETCURSEL, 0, 0);
          return TRUE;
        }
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNSAVEART); //IDD_WINVNSAVEART
		break;


  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) 
		{


        case IDOK:
          {
            HWND hList = GetDlgItem(hDlg, IDD_FILENAME);
            int i = 1, j = 0;
            GetDlgItemText (hDlg, IDD_FILENAME, SaveArtFileName, MAXFILENAME - 1);
            if (*SaveArtFileName == '\0') {
                  WVMessageBox (hDlg, IDS_WVCNCM_ENTER_FILENAME, IDS_WVCNCM_FILENAME_ERROR, MB_OK | MB_ICONSTOP);
                  break;
            }

            WritePrivateProfileString(MRUENTRIES, "file01", SaveArtFileName, szAppProFile);
            while ( j < MAX_DIR_ENTRIES)
            {
              if (SendMessage (hList, CB_GETLBTEXT, j, (LONG) (LPSTR) fileName) != CB_ERR)
              {
                if (stricmp(fileName, SaveArtFileName))
                {
                  char szEntry[20];
                  wsprintf(szEntry, "file%02d", ++i);
                  
                  WritePrivateProfileString(MRUENTRIES, szEntry, fileName, szAppProFile);
                }
              }
              j++;  
            } 

            SaveArtAppend = MyAppend;
            if ( MRRWriteDocument (ActiveArticleDoc, sizeof (TypText), SaveArtFileName, SaveArtAppend) != NO_ERROR) 
			{
                  WVMessageBox (NetDoc.hDocWnd, IDS_WVCNCM_WRITE_ERROR, IDS_WVCNCM_PROBLEMS_SAVING, MB_OK | MB_ICONEXCLAMATION);
                  *SaveArtFileName = '\0';
            }
            EndDialog (hDlg, TRUE);
          }
          break;

        case ID_BROWSE:
          {
            char szDir[MAXFILENAME];
            fileName[0] = '\0';
            GetDlgItemText (hDlg, IDD_FILENAME, szDir, MAXFILENAME - 1);
          
            if (AskForNewFileName (hDlg, fileName, szDir, MyAppend) == SUCCESS)
                  SetDlgItemText (hDlg, IDD_FILENAME, fileName);
          }
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case IDD_APPEND:
          MyAppend = !MyAppend;
          CheckDlgButton (hDlg, IDD_APPEND, MyAppend);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*-- function WinVnSaveArtsDlg ---------------------------------------
 *
 *  Dialog function to handle Save Articles dialog box.
 *  Same as Save Article dialog, except actual save is deferred until
 *  later (When IDM_RETRIEVE_COMPLETE Message gets sent to Group win)
 *  (JSC)
 */

BOOL CALLBACK WinVnSaveArtsDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  static int MyAppend;
  char fileName[MAXFILENAME];
  OFSTRUCT ofs;
  HFILE hFile;
  UINT mode;

  switch (iMessage) {

  case WM_INITDIALOG:
        {
          int i;
          char szEntry[20];
          HWND hList = GetDlgItem(hDlg, IDD_FILENAME);

          MyAppend = SaveArtAppend;
          CheckDlgButton (hDlg, IDD_APPEND, MyAppend);
          CheckDlgButton (hDlg, IDD_KEEP_HEADER_VISIBLE, KeepArticleHeaderVisible);
          SetDlgItemText (hDlg, IDD_FILENAME, SaveArtFileName);


          for (i = MAX_DIR_ENTRIES; i > 0; i--)
          {
              wsprintf(szEntry, "file%02d", i);
              GetPrivateProfileString (MRUENTRIES, szEntry, "", fileName, MAXFILENAME, szAppProFile);
              if (*fileName)
                SendMessage (hList, CB_INSERTSTRING, 0, (LONG) (LPSTR) fileName);

          }
          if (*SaveArtFileName)
            SetDlgItemText (hDlg, IDD_FILENAME, SaveArtFileName);
          else
            SendMessage(hList, CB_SETCURSEL, 0, 0);
          
          return TRUE;
        }
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNSAVEARTS); //IDD_WINVNSAVEARTS
		break;


  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {

        case IDOK:
          {
            HWND hList = GetDlgItem(hDlg, IDD_FILENAME);
            int i = 1, j = 0;

            GetDlgItemText (hDlg, IDD_FILENAME, fileName, MAXFILENAME - 1);
            if (*fileName == '\0') {
                  WVMessageBox (hDlg, IDS_WVCNCM_WRITE_ERROR, IDS_WVCNCM_PROBLEMS_SAVING, MB_OK | MB_ICONSTOP);
                  break;
            }
            mode = (OpenFile (fileName, &ofs, OF_EXIST) == HFILE_ERROR) ? OF_CREATE : OF_WRITE;
            if ((hFile = OpenFile (fileName, &ofs, mode)) == HFILE_ERROR) {
                  WVArgMessageBox (hDlg, IDS_WVCNCM_WRITE_ERROR_ARG, IDS_WVCNCM_PROBLEMS_SAVING, MB_OK | MB_ICONSTOP, fileName);
                  break;
            }
            _lclose (hFile);
            strcpy (SaveArtFileName, fileName);

            WritePrivateProfileString(MRUENTRIES, "file01", SaveArtFileName, szAppProFile);
            while ( j < MAX_DIR_ENTRIES)
            {
              if (SendMessage (hList, CB_GETLBTEXT, j, (LONG) (LPSTR) fileName) != CB_ERR)
              {
                if (stricmp(fileName, SaveArtFileName))
                {
                  char szEntry[20];
                  wsprintf(szEntry, "file%02d", ++i);
                  
                  WritePrivateProfileString(MRUENTRIES, szEntry, fileName, szAppProFile);
                }
              }
              j++;  
            } 

            KeepArticleHeaderVisible = IsDlgButtonChecked (hDlg, IDD_KEEP_HEADER_VISIBLE);
            SaveArtAppend = MyAppend;
            EndDialog (hDlg, TRUE);

          }
          break;

        case ID_BROWSE:
          {
            char szDir[MAXFILENAME];
            fileName[0] = '\0';
            GetDlgItemText (hDlg, IDD_FILENAME, szDir, MAXFILENAME - 1);
          
            if (AskForNewFileName (hDlg, fileName, szDir, MyAppend) == SUCCESS)
                  SetDlgItemText (hDlg, IDD_FILENAME, fileName);
          }
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case IDD_APPEND:
          MyAppend = !MyAppend;
          CheckDlgButton (hDlg, IDD_APPEND, MyAppend);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*-- function WinVnDecodeArtsDlg ---------------------------------------
 *
 *  Dialog function to handle Decode Articles dialog box.
 *  (JSC)
 */

BOOL CALLBACK WinVnDecodeArtsDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  int len;

  switch (iMessage) {

  case WM_INITDIALOG:
        SetDlgItemText (hDlg, IDD_DECODE_PATH, DecodePathName);
        CheckDlgButton (hDlg, IDD_EXECUTE_DECODED, ExecuteDecodedFiles);
        CheckDlgButton (hDlg, IDD_KEEP_HEADER_VISIBLE, KeepArticleHeaderVisible);
        CheckDlgButton (hDlg, IDD_VERBOSE_STATUS, CodingStatusVerbose);
        CheckDlgButton (hDlg, IDD_MINIMIZE_CODING_STATUS, MinimizeStatusWindows);
        CheckDlgButton (hDlg, IDD_DUMB_DECODE, DumbDecode);
        CheckDlgButton (hDlg, IDD_INCLUDE_OPEN_ARTS, AlsoDecodeOpenArticles);

        EnableWindow (GetDlgItem (hDlg, IDD_INCLUDE_OPEN_ARTS), (BOOL) (NumArticleWnds > 0));
        EnableWindow (GetDlgItem (hDlg, IDD_KEEP_HEADER_VISIBLE), (BOOL) lParam);
        if (lParam)
          WVSetWindowText (hDlg, IDS_WVCNCM_DECODE1);
        else
          WVSetWindowText (hDlg, IDS_WVCNCM_DECODE2);
        break;

	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNDECODEARTS);
		break;
  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
          AlsoDecodeOpenArticles = (IsDlgButtonChecked (hDlg, IDD_INCLUDE_OPEN_ARTS) != 0);
          GetDlgItemText (hDlg, IDD_DECODE_PATH, DecodePathName, MAXFILENAME - 1);
          RemoveTrailingWhiteSpace (DecodePathName);
          len = strlen (DecodePathName);
          if (len == 0) {
                WVMessageBox (hDlg, IDS_WVCNCM_ENTER_PATH, IDS_WVCNCM_DECODE_PATH, MB_OK);
                break;
          }

          /* allow path like c:\ -- but otherwise trim off trailing slashes */
          if ((len != 3 || DecodePathName[1] != ':' || DecodePathName[2] != '\\') &&
                  DecodePathName[len - 1] == '\\') {
                DecodePathName[len - 1] = '\0';
          }

          if (_access (DecodePathName, 0) < 0) {
                WVArgMessageBox (hDlg, IDS_WVCNCM_NO_PATH, IDS_WVCNCM_DECODE_PATH, MB_OK | MB_ICONSTOP, DecodePathName);
                break;
          }

          ExecuteDecodedFiles = (IsDlgButtonChecked (hDlg, IDD_EXECUTE_DECODED) != 0);
          KeepArticleHeaderVisible = IsDlgButtonChecked (hDlg, IDD_KEEP_HEADER_VISIBLE);
          CodingStatusVerbose = IsDlgButtonChecked (hDlg, IDD_VERBOSE_STATUS);
          MinimizeStatusWindows = IsDlgButtonChecked (hDlg, IDD_MINIMIZE_CODING_STATUS);
          DumbDecode = IsDlgButtonChecked (hDlg, IDD_DUMB_DECODE);
          EndDialog (hDlg, TRUE);
          break;

        case ID_BROWSE:
          if (AskForFilePath (hDlg, DecodePathName, "Select Decode Path") == SUCCESS)
                SetDlgItemText (hDlg, IDD_DECODE_PATH, DecodePathName);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case IDD_SMART_FILER:
          DialogBox (hInst, MAKEINTRESOURCE(IDD_WINVNSMARTFILER), hDlg, lpfnWinVnSmartFilerDlg);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}


void SelectTypeByExtension (HWND hListBox, char *fileName)
{
  char temp[MAXFILENAME];

  GetFileExtension (temp, fileName);
  if (!_stricmp (temp, "gif"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Image/GIF");
  else if (!_stricmp (temp, "jpg") || !_strnicmp (temp, "jpe", 3))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Image/JPEG");
  else if (!_stricmp (temp, "zip"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Application/Zip");
  else if (!_stricmp (temp, "mpg") || !_strnicmp (temp, "mpe", 3))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Video/MPEG");
  else if (!_stricmp (temp, "avi"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Video/AVI");
  else if (!_stricmp (temp, "ps"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Application/PostScript");
  else if (!_stricmp (temp, "txt") || !_stricmp (temp, "bat") || !_stricmp (temp, "c") || !_stricmp (temp, "cpp") || !_stricmp (temp, "h"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Text/Plain");
  else if (!_stricmp (temp, "wav"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Audio/x-wav");
  else if (!_stricmp (temp, "mid"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Audio/midi");
  else if (!_stricmp (temp, "doc"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Application/msword");
  else if (!_stricmp (temp, "rtf"))
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Application/rtf");
  else
        SendMessage (hListBox, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) DefaultContentType);
}

/*-- function WinVnAttachDlg ---------------------------------------
 *
 *  Dialog function to handle attachments to postings
 *  pointer to attachment struct should be passed in lParam
 *  jsc 9/24/94
 */

BOOL CALLBACK WinVnAttachDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  static TypAttachment *thisAttach;
  int i, select;
//  extern char *ContentTypes[NUM_CONTENT_TYPES];
//  extern char *EncodingTypes[NUM_ENCODING_TYPES];

  switch (iMessage) {
  case WM_INITDIALOG:
        thisAttach = (TypAttachment *) lParam;

        for (i = 0; i < NUM_CONTENT_TYPES; i++)
          SendDlgItemMessage (hDlg, IDD_CONTENT_TYPE,
                                                  CB_ADDSTRING, 0, (LPARAM) (LPSTR) ContentTypes[i]);

        // if DefaultContentType is not "other", add the type to the list
        if (*DefaultContentType && _stricmp (DefaultContentType, "Other")) {
          SendDlgItemMessage (hDlg, IDD_CONTENT_TYPE,
                                          CB_ADDSTRING, 0, (LPARAM) (LPSTR) DefaultContentType);
          SendDlgItemMessage (hDlg, IDD_CONTENT_TYPE, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) DefaultContentType);
        }
        else {
          SendDlgItemMessage (hDlg, IDD_CONTENT_TYPE, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) "Other");
        }
        CheckDlgButton (hDlg, IDD_MAKE_DEFAULT_CONTENT, 0);
        CheckDlgButton (hDlg, IDD_MAKE_DEFAULT_ENCODING, 0);

        if (*(thisAttach->encodingType))
          SetupEncodingOptions (hDlg, thisAttach->encodingType, FALSE, TRUE);
        else
          SetupEncodingOptions (hDlg, DefaultEncodingType, FALSE, TRUE);

        if (*(thisAttach->fileName))
          SetDlgItemText (hDlg, IDD_FILENAME, thisAttach->fileName);

        if (*(thisAttach->contentType))
          SetDlgItemText (hDlg, IDD_CONTENT_TYPE, (LPCSTR) thisAttach->contentType);
        else if (*(thisAttach->fileName))
          SelectTypeByExtension (GetDlgItem (hDlg, IDD_CONTENT_TYPE), thisAttach->fileName);

        CheckDlgButton (hDlg, IDD_ATTACH_NEXT, thisAttach->attachInNewArt);
        break;

	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNATTACH);
		break;

  case WM_COMMAND:
        switch (LOWORD (wParam)) 
		{

		case IDD_FILENAME:
          if (GET_WM_COMMAND_CMD (wParam, lParam) == EN_KILLFOCUS) {
                GetDlgItemText (hDlg, IDD_CONTENT_TYPE, (LPSTR) str, MAXINTERNALLINE);
                if (*str == '\0') {
                  GetDlgItemText (hDlg, IDD_FILENAME, str, MAXINTERNALLINE);
                  SelectTypeByExtension (GetDlgItem (hDlg, IDD_CONTENT_TYPE), str);
                }
          }
          break;

        case IDOK:
          GetDlgItemText (hDlg, IDD_FILENAME, thisAttach->fileName, MAXINTERNALLINE);
          if (*(thisAttach->fileName) == '\0') {
                WVMessageBox (hDlg, IDS_WVCNCM_ENTER_ATTACH, IDS_WVCNCM_FILENAME_ERROR, MB_OK);
                break;
          }
//              if (_access(thisAttach->fileName, 0) < 0) {
          if ((thisAttach->numBytes = GetFileLength (thisAttach->fileName)) == 0) {
                WVArgMessageBox (hDlg, IDS_WVCNCM_ATTACH_ERROR, IDS_WVCNCM_ATTACH_FILE, MB_OK | MB_ICONSTOP, thisAttach->fileName);
                break;
          }

          GetDlgItemText (hDlg, IDD_CONTENT_TYPE, (LPSTR) thisAttach->contentType, MAXINTERNALLINE);
          if (IsDlgButtonChecked (hDlg, IDD_MAKE_DEFAULT_CONTENT))
                strcpy (DefaultContentType, thisAttach->contentType);

          select = (int) SendDlgItemMessage (hDlg, IDD_CODING_TYPE, CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0);
          if (select != CB_ERR)
                SendDlgItemMessage (hDlg, IDD_CODING_TYPE,
                                                        CB_GETLBTEXT, (WPARAM) select, (LPARAM) (LPSTR) thisAttach->encodingType);
          if (IsDlgButtonChecked (hDlg, IDD_MAKE_DEFAULT_ENCODING))
                strcpy (DefaultEncodingType, thisAttach->encodingType);

          thisAttach->attachInNewArt = IsDlgButtonChecked (hDlg, IDD_ATTACH_NEXT);

		  //TODO: Remove when supported
		  if ( EncodingTypeToNum (thisAttach->encodingType) == CODE_YENC)
		  {
			WVMessageBox(hDlg, "yEnc encoding support not finished yet.", "Sorry", MB_OK);
			break;
		  }

          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case ID_BROWSE:
          if (AskForExistingFileName (hDlg, thisAttach->fileName, "Open File To Attach") == SUCCESS) {
                SetDlgItemText (hDlg, IDD_FILENAME, thisAttach->fileName);
                SelectTypeByExtension (GetDlgItem (hDlg, IDD_CONTENT_TYPE), thisAttach->fileName);
          }
          break;

        default:
          return FALSE;
        }

  default:
        return FALSE;
  }
  return TRUE;
}

/*-- function WinVnAttachPrefsDlg ---------------------------------------
 *
 *  Dialog function to handle attachment preferences
 *  jsc 9/24/94
 */

BOOL CALLBACK WinVnAttachPrefsDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  char temp[MAXINTERNALLINE];
  extern char *GetFileExtension ();
  extern BOOL isnumber (char *str);
  register int i;
  int genMIME, result, num;
  extern char *ContentTypes[NUM_CONTENT_TYPES];
  long newLength, multiplier;
  BOOL bSel = FALSE;

  switch (iMessage) {
  case WM_INITDIALOG:
        CheckDlgButton (hDlg, IDD_ATTACH_NEXT, DefaultAttachInNewArt);

        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LENGTH,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "32000");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LENGTH,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "60000");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LENGTH,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "100000");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LENGTH,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "157500");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LENGTH,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "315000");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LENGTH,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "472500");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LENGTH,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "None");

        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LINES,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "1000");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LINES,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "2500");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LINES,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "5000");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LINES,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "7500");
        SendDlgItemMessage (hDlg, IDD_ARTICLE_SPLIT_LINES,
                                                CB_ADDSTRING, 0, (LPARAM) (LPSTR) "None");

        num = SetupEncodingOptions (hDlg, DefaultEncodingType, TRUE, TRUE);

        if (ArticleSplitLength == 0) 
		{
          SetDlgItemText (hDlg, IDD_ARTICLE_SPLIT_LENGTH, "None");
          SetDlgItemText (hDlg, IDD_ARTICLE_SPLIT_LINES, "None");
        }
        else 
		{
			SetDlgItemText (hDlg, IDD_ARTICLE_SPLIT_LENGTH, ltoa (ArticleSplitLength, str, 10));
			multiplier = 63;
			if (num == CODE_YENC)
				multiplier = 130;
			newLength = ArticleSplitLength / multiplier;
			SetDlgItemText (hDlg, IDD_ARTICLE_SPLIT_LINES, ltoa (newLength, str, 10));

        }

		





        SetDlgItemText (hDlg, IDD_SUBJECT_TEMPLATE, SubjectTemplate);

        CheckDlgButton (hDlg, IDD_GENERATE_MIME, GenerateMIME);
        if (!GenerateMIME) {
//        EnableWindow (GetDlgItem (hDlg, IDD_MIME_BOUNDARY), FALSE);
          EnableWindow (GetDlgItem (hDlg, IDD_CONTENT_TYPE), FALSE);
        }
        CheckDlgButton (hDlg, IDD_MIME_USAGE_SUGGESTIONS, MIMEUsageSuggestions);

        for (i = 0; i < NUM_CONTENT_TYPES; i++)
          SendDlgItemMessage (hDlg, IDD_CONTENT_TYPE,
                              CB_ADDSTRING, 0, (LPARAM) (LPSTR) ContentTypes[i]);

        if (_stricmp (DefaultContentType, "Other"))
          SendDlgItemMessage (hDlg, IDD_CONTENT_TYPE,
                                          CB_ADDSTRING, 0, (LPARAM) (LPSTR) DefaultContentType);

//      SendDlgItemMessage (hDlg, IDD_MIME_BOUNDARY, EM_LIMITTEXT, (WPARAM) MAXBOUNDARYLINE, 0L);
//      SetDlgItemText (hDlg, IDD_MIME_BOUNDARY, MIMEBoundary);
        return TRUE;
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)410); //IDD_WINVNATTACHPREFS
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
          DefaultAttachInNewArt = IsDlgButtonChecked (hDlg, IDD_ATTACH_NEXT);
          if (ProcessEncodingOptions (hDlg, DefaultEncodingType) == FAIL)
                break;

          GetDlgItemText (hDlg, IDD_ARTICLE_SPLIT_LENGTH, temp, MAXINTERNALLINE);
          if (!_stricmp (temp, "None")) 
		  {
                ArticleSplitLength = 0;
          }
          else 
		  {
                if (isnumber (temp)) 
				{
                  newLength = atol (temp);
                }
                else 
				{
                  WVMessageBox (hDlg, IDS_WVCNCM_SPLIT_HELP, IDS_WVCNCM_SPLIT_LEN_ERROR, MB_OK);
                  break;
                }
                if (newLength > 0L && newLength < 4096L)  // allow 0
				{      

                  WVMessageBox (hDlg, IDS_WVCNCM_SPLIT_4K, IDS_WVCNCM_ART_LEN_ERROR, MB_OK | MB_ICONSTOP);
                  break;
                }
                //if (newLength > 60000L &&
                //    WVMessageBox (hDlg, IDS_WVCNCM_ART_60K, IDS_WVCNCM_ART_LEN_WARN, MB_YESNO | MB_ICONINFORMATION) == IDNO) {
                //  break;
                //}
                if (newLength > 499000L &&
                    WVMessageBox (hDlg, IDS_WVCNCM_ART_500K, IDS_WVCNCM_ART_LEN_WARN, MB_YESNO | MB_ICONINFORMATION) == IDNO) {
                  break;
                }

                ArticleSplitLength = newLength;
          }

          GetDlgItemText (hDlg, IDD_SUBJECT_TEMPLATE, SubjectTemplate, MAXINTERNALLINE);

          GenerateMIME = IsDlgButtonChecked (hDlg, IDD_GENERATE_MIME);
//        GetDlgItemText (hDlg, IDD_MIME_BOUNDARY, MIMEBoundary, MAXBOUNDARYLEN);

          // *** should check here that characters in boundary are all valid
          GetDlgItemText (hDlg, IDD_CONTENT_TYPE, (LPSTR) DefaultContentType, MAXINTERNALLINE);

          MIMEUsageSuggestions = IsDlgButtonChecked (hDlg, IDD_MIME_USAGE_SUGGESTIONS);
          if (MIMEUsageSuggestions) {
                num = EncodingTypeToNum (DefaultEncodingType);
                if (GenerateMIME && num != CODE_NONE && num != CODE_BASE64) {
                  result = WVArgMessageBox (hDlg, IDS_WVCNCM_DISABLE_MIME, IDS_WVCNCM_MIME_USE,
                                            MB_YESNOCANCEL | MB_ICONINFORMATION, DefaultEncodingType );
                  if (result == IDYES)
                        GenerateMIME = FALSE;
                  else if (result == IDCANCEL)
                        break;

                }
                if (!GenerateMIME && num == CODE_BASE64) {
                  result = WVMessageBox (hDlg, IDS_WVCNCM_ENABLE_MIME, IDS_WVCNCM_MIME_USE, MB_YESNOCANCEL | MB_ICONINFORMATION);
                  if (result == IDYES)
                        GenerateMIME = TRUE;
                  else if (result == IDCANCEL)
                        break;
                }
          }

		  //TODO: Remove when supported
		  if ( EncodingTypeToNum (DefaultEncodingType) == CODE_YENC)
		  {
			WVMessageBox(hDlg, "yEnc encoding support not finished yet.", "Sorry", MB_OK);
			break;
		  }

          EndDialog (hDlg, TRUE);
          break;

        case IDD_GENERATE_MIME:
          genMIME = (BOOL) IsDlgButtonChecked (hDlg, IDD_GENERATE_MIME);
//        EnableWindow (GetDlgItem (hDlg, IDD_MIME_BOUNDARY), genMIME);
          EnableWindow (GetDlgItem (hDlg, IDD_CONTENT_TYPE), genMIME);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;
		case IDD_CODING_TYPE:
		case IDD_ARTICLE_SPLIT_LENGTH:
		case IDD_ARTICLE_SPLIT_LINES:
			bSel = (GET_WM_COMMAND_CMD (wParam, lParam) == CBN_SELENDOK);
			
			if ( bSel || GET_WM_COMMAND_CMD (wParam, lParam) == CBN_KILLFOCUS ||
						 GET_WM_COMMAND_CMD (wParam, lParam) == CBN_EDITCHANGE ) 
			{
				if (GetDlgItemText (hDlg, IDD_CODING_TYPE, temp, MAXINTERNALLINE) )
				{
					
					UINT nControlID = GET_WM_COMMAND_ID(wParam, lParam);
					UINT nOtherControl = IDD_ARTICLE_SPLIT_LINES;

					if (nControlID == IDD_CODING_TYPE)
					{
						bSel = FALSE;
						nControlID = IDD_ARTICLE_SPLIT_LENGTH;
						
						num = (int) SendDlgItemMessage (hDlg, IDD_CODING_TYPE, CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0);
						if (num != CB_ERR)
							SendDlgItemMessage (hDlg, IDD_CODING_TYPE, CB_GETLBTEXT, (WPARAM) num, (LPARAM) (LPSTR) temp);
					}

					if (nControlID == IDD_ARTICLE_SPLIT_LINES )
						nOtherControl = IDD_ARTICLE_SPLIT_LENGTH;

					num = EncodingTypeToNum (temp);
					multiplier = 63;
					if (num == CODE_YENC)
						multiplier = 130;
					
					*temp = '\0';
					newLength = 0;

					if (bSel)
					{
						num = (int) SendDlgItemMessage (hDlg, nControlID, CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0);
						if (num != CB_ERR)
							SendDlgItemMessage (hDlg, nControlID, CB_GETLBTEXT, (WPARAM) num, (LPARAM) (LPSTR) temp);
					}
					else
					{
						GetDlgItemText (hDlg, nControlID, temp, MAXINTERNALLINE);
					}

					if (isnumber (temp)) 
					{
						newLength = atol (temp);
						if (newLength > 0)
						{
							if (nControlID == IDD_ARTICLE_SPLIT_LENGTH)
							{
								newLength /= multiplier;
								if (newLength == 0)
									newLength = 1;
							}
							else
								newLength *= multiplier;
							SetDlgItemText (hDlg, nOtherControl, ltoa (newLength, temp, 10));
						}
						else
						{
							SetDlgItemText (hDlg, nOtherControl, "None");
						}
					}
							 

				}

			}

			break;


        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}
/*-- function WinVnEncodeDlg ---------------------------------------
 *
 *  Dialog function to handle encoding to a file
 *  (JSC)
 */

BOOL CALLBACK WinVnEncodeDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  static TypAttachment *thisAttach;

  switch (iMessage) {
  case WM_INITDIALOG:
        thisAttach = (TypAttachment *) lParam;
        // include custom, but do not show "none" in the type-list
        SetupEncodingOptions (hDlg, DefaultEncodingType, TRUE, FALSE);
        return TRUE;
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNENCODE);
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {

        case IDOK:
          GetDlgItemText (hDlg, IDD_FILENAME, thisAttach->fileName, MAXFILENAME - 1);
          if (*(thisAttach->fileName) == '\0') {
                WVMessageBox (hDlg, IDS_WVCNCM_ENTER_ENCODE, IDS_WVCNCM_FILENAME_ERROR, MB_OK);
                break;
          }

          if (_access (thisAttach->fileName, 0) < 0) {
                WVArgMessageBox (hDlg, IDS_WVCNCM_BAD_FILE, IDS_WVCNCM_FILENAME_ERROR, MB_OK | MB_ICONSTOP,thisAttach->fileName);
                break;
          }

          if (ProcessEncodingOptions (hDlg, thisAttach->encodingType) == FAIL)
                break;

          if (IsDlgButtonChecked (hDlg, IDD_MAKE_DEFAULT_ENCODING))
                strcpy (DefaultEncodingType, thisAttach->encodingType);

		  //TODO: Remove when supported
		  if ( EncodingTypeToNum (thisAttach->encodingType) == CODE_YENC)
		  {
			WVMessageBox(hDlg, "yEnc encoding support not finished yet.", "Sorry", MB_OK);
			break;
		  }

          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case ID_BROWSE:
          if (AskForExistingFileName (hDlg, thisAttach->fileName, "Open File To Be Encoded") == SUCCESS)
                SetDlgItemText (hDlg, IDD_FILENAME, thisAttach->fileName);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*----------------------------------------------------------------------
 * Encoding options functions for use by two dialogs
 * wvattachdlg and wvencodedlg
 * (JSC)
 */
int SetupEncodingOptions (HWND hDlg, char *startEncoding, BOOL includeCustom, BOOL showNone)
{
  register int i;
  char temp[MAXENCODINGTYPELEN];
  int nType = CODE_UNKNOWN;

  if (*startEncoding == '\0')
        startEncoding = DefaultEncodingType;

  // This combo box is NOSORT, so entries can be indexed in order they 
  // added here. But BE CAREFUL!
  for (i = 0; i < NUM_ENCODING_TYPES; i++) {
        if (showNone || _stricmp (EncodingTypes[i], "None")) {
          SendDlgItemMessage (hDlg, IDD_CODING_TYPE,
                              CB_ADDSTRING, 0, (LPARAM) (LPSTR) EncodingTypes[i]);
        }
  }

  nType = EncodingTypeToNum (startEncoding);
  if (nType == CODE_UNKNOWN)
        strcpy (temp, "None");
  else
        strcpy (temp, startEncoding);

  SendDlgItemMessage (hDlg, IDD_CODING_TYPE, CB_SELECTSTRING, (WPARAM) - 1, (LPARAM) (LPSTR) temp);

  if (includeCustom && UserCodingTable[0] != '\0')
        SetDlgItemText (hDlg, IDD_CUSTOM_TABLE, UserCodingTable);
  
  return nType;
}

BOOL ProcessEncodingOptions (HWND hDlg, char *newEncodingType)
{
  char temp[MAXINTERNALLINE];
  int select, x;
  char dummyMap[128];

  select = (int) SendDlgItemMessage (hDlg, IDD_CODING_TYPE, CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0);
  if (select != CB_ERR)
        SendDlgItemMessage (hDlg, IDD_CODING_TYPE,
                            CB_GETLBTEXT, (WPARAM) select, (LPARAM) (LPSTR) temp);

  if (!_stricmp (temp, "Custom"))
        if (GetDlgItemText (hDlg, IDD_CUSTOM_TABLE, UserCodingTable, CODINGTABLESIZE + 1)) {
          if (strlen (UserCodingTable) != CODINGTABLESIZE) {
                WVMessageBox (hDlg, IDS_WVCNCM_INVALID_TABLE, IDS_WVCNCM_CUSTOM_CODING, MB_OK);
                return (FAIL);
          }
          if ((x = CreateCodingMap (dummyMap, UserCodingTable)) != -1) {
                sprintf (temp, "%c", x);
                WVArgMessageBox (hDlg, IDS_WVCNCM_DUPLICATE_TABLE, IDS_WVCNCM_CUSTOM_CODING, MB_OK, temp);
                return (FAIL);
          }
        }
        else {
          WVMessageBox (hDlg, IDS_WVCNCM_CODING_HELP, IDS_WVCNCM_CUSTOM_CODING, MB_OK);
          return (FAIL);
        }

  strcpy (newEncodingType, temp);

  return (SUCCESS);
}


/*-- function WinVnExitDlg---------------------------------------
 *
 *  Dialog function to handle exit from WinVn
 *  Sets globals SaveNewsrc and SaveConfig - leaves actual saving up to caller
 *  (JSC 1/8/94)
 */

BOOL CALLBACK WinVnExitDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  switch (iMessage) {
  case WM_INITDIALOG:
        CheckDlgButton (hDlg, IDD_SAVE_NEWSRC, 1);
        CheckDlgButton (hDlg, IDD_SAVE_CONFIG, 1);
        return TRUE;
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNEXIT);
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
          SaveNewsrc = IsDlgButtonChecked (hDlg, IDD_SAVE_NEWSRC);
          SaveConfig = IsDlgButtonChecked (hDlg, IDD_SAVE_CONFIG);
          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*-- function WinVnFindDlg ---------------------------------------
 *
 *  Dialog function to handle Search dialog box.
 */

BOOL CALLBACK WinVnFindDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
//  static int MyAppend;

  switch (iMessage) {

  case WM_INITDIALOG:
        SetDlgItemText (hDlg, IDD_SEARCH_STRING, FindDoc->SearchStr);
        return TRUE;
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNFIND);
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
          GetDlgItemText (hDlg, IDD_SEARCH_STRING, FindDoc->SearchStr, MAXFINDSTRING - 1);

          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*--- Function WinVnPersonalInfoDlg ----------------------------------
 *
 *  Dialog function to obtain personal configuration info
 *  (Name, email address, etc.) from the user.
 */

BOOL CALLBACK WinVnPersonalInfoDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  switch (iMessage) {
  case WM_INITDIALOG:
        SetDlgItemText (hDlg, ID_CONFIG_EMAIL, MailAddress);
        SetDlgItemText (hDlg, ID_CONFIG_NAME, UserName);
        SetDlgItemText (hDlg, IDD_REPLY_TO, ReplyTo);
        SetDlgItemText (hDlg, ID_CONFIG_ORG, Organization);
        return TRUE;
        break;

	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNPERSONAL);
		break;


  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
          GetDlgItemText (hDlg, ID_CONFIG_EMAIL, MailAddress, MAILLEN - 1);
          GetDlgItemText (hDlg, ID_CONFIG_NAME, UserName, MAILLEN - 1);
          GetDlgItemText (hDlg, IDD_REPLY_TO, ReplyTo, MAILLEN - 1);
          GetDlgItemText (hDlg, ID_CONFIG_ORG, Organization, MAILLEN - 1);

          if (!strcmp (MailAddress, ReplyTo)) {         /* clear identical reply-to */
                WVMessageBox(hDlg, IDS_WVCNCM_NO_REPLYTO, IDS_WVCNCM_CLEAR_REPLYTO, MB_OK);
                *ReplyTo = '\0';
                SetDlgItemText(hDlg, IDD_REPLY_TO, "");
                break;
          }

          if (!ValidEmailAddress(MailAddress)) {          /* Validate E-mail address */
                WVMessageBox(hDlg, IDS_WVCNCM_INVALID_EMAIL_HELP, IDS_WVCNCM_INVALID_EMAIL, MB_OK);
        //      *MailAddress = '\0';
                break;
          }

          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

// Configure Execute Dialog for setting up Universal Resource Locator (URL)
// interface between WinVN and World Wide Web (WWW) or other browsers.
//
//                                                     JD 7/23/95

BOOL CALLBACK WinVnExecuteInfoDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  char *dlgname;
  int id;
  char temp[MAXFILENAME], browserpath[MAXFILENAME];

  switch (iMessage) {
  case WM_INITDIALOG:
        SetDlgItemText (hDlg, ID_CONFIG_URL_HTTP,HttpClient);
        SetDlgItemText (hDlg, ID_CONFIG_URL_FTP,FtpClient);
        SetDlgItemText (hDlg, ID_CONFIG_URL_GOPHER,GopherClient);
        SetDlgItemText (hDlg, ID_CONFIG_URL_WAIS, WaisClient);
        SetDlgItemText (hDlg, IDC_CONFIG_URL_DDE_ID_STRING, DdeServName);
        CheckDlgButton (hDlg, IDC_CONFIG_URL_DDE, UsingDDE);
        CheckRadioButton (hDlg, IDC_CONFIG_URL_DDE_RADIO, IDC_CONFIG_URL_DDE_RADIO4,
                                        DdeServID + IDC_CONFIG_URL_DDE_RADIO);
        EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO),  UsingDDE);
        EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO1), UsingDDE);
        EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO2), UsingDDE);
        EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO3), UsingDDE);
        EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO4), UsingDDE);
        if (IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE_RADIO1) != 0)
			EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_ID_STRING), UsingDDE);
        return TRUE;
        break;

  	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD) 416); //IDD_WINVNEXECUTE
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
                
          GetDlgItemText (hDlg, ID_CONFIG_URL_HTTP, HttpClient, MAXFILENAME-1);
          GetDlgItemText (hDlg, ID_CONFIG_URL_FTP, FtpClient, MAXFILENAME-1);
          GetDlgItemText (hDlg, ID_CONFIG_URL_GOPHER, GopherClient, MAXFILENAME-1);
          GetDlgItemText (hDlg, ID_CONFIG_URL_WAIS, WaisClient, MAXFILENAME-1);
          UsingDDE = (IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE) != 0);
          if (UsingDDE) {
                DdeServID = IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE_RADIO1) ? 1 :
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE_RADIO2) ? 2 : 
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE_RADIO3) ? 3 :
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE_RADIO4) ? 4 : 0;

        if (IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE_RADIO1) != 0)
                   GetDlgItemText (hDlg, IDC_CONFIG_URL_DDE_ID_STRING , DdeServName, MAXFILENAME-1);
          }
          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case IDC_CONFIG_URL_DDE:
          EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO),
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE) != 0);
          EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO1),
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE) != 0);
          EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO2),
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE) != 0);
          EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO3),
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE) != 0);
          EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_RADIO4),
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE) != 0);
          EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_ID_STRING),
                                        (IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE) != 0 &&
                                        IsDlgButtonChecked (hDlg, IDC_CONFIG_URL_DDE_RADIO1) != 0));
          return FALSE;

        case IDC_CONFIG_URL_DDE_RADIO4:
        case IDC_CONFIG_URL_DDE_RADIO2:
        case IDC_CONFIG_URL_DDE_RADIO3:
        case IDC_CONFIG_URL_DDE_RADIO:
			EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_ID_STRING), FALSE);
          return FALSE;

        case IDC_CONFIG_URL_DDE_RADIO1:
			EnableWindow (GetDlgItem (hDlg, IDC_CONFIG_URL_DDE_ID_STRING), TRUE);
          return FALSE;

        case IDC_CONFIG_URL_BUTTON1:
                dlgname = "Select WWW Browser";
                id = ID_CONFIG_URL_HTTP;
                goto ExecBrowse;
        case IDC_CONFIG_URL_BUTTON2:
                dlgname = "Select FTP Browser";
                id = ID_CONFIG_URL_FTP;
                goto ExecBrowse;
        case IDC_CONFIG_URL_BUTTON3:
                dlgname = "Select Gopher Browser";
                id = ID_CONFIG_URL_GOPHER;
                goto ExecBrowse;
        case IDC_CONFIG_URL_BUTTON4:
                dlgname = "Select WAIS Browser";
                id = ID_CONFIG_URL_WAIS;
ExecBrowse:
          if (AskForExistingFileName (hDlg, temp, dlgname) == SUCCESS) {
                if (strchr(temp, ' ')) {
                        sprintf(browserpath, "\"%s\" %%u", temp);
                } else {
                        sprintf(browserpath, "%s %%u", temp);
                }
                SetDlgItemText (hDlg, id, browserpath);
          }
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*--- Function WinVnComposePrefsDlg ---------------------------------------------
 *
 * Composition prefs
 *
 * jsc 9/28/94
 */

BOOL CALLBACK WinVnComposePrefsDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  static int MyShowFrom, MyShowOrg, MyShowReplyTo, MyShowKeywords, MyShowSummary,
    MyShowDist, MyShowFollowupTo;
  char temp[MAXFILENAME];

  switch (iMessage) {
  case WM_INITDIALOG:
#ifdef RFC1522
        CheckDlgButton (hDlg, IDD_ENCODE_RFC1522, EncodeRFC1522);
#else
        EnableWindow(GetDlgItem (hDlg, IDD_ENCODE_RFC1522), FALSE);          
#endif  
        CheckDlgButton (hDlg, IDD_WORD_WRAP, WordWrap);
        CheckDlgButton (hDlg, IDD_PREFILL_CC, PrefillCcAddress);
        CheckDlgButton (hDlg, IDD_SHOW_FROM_HDR, ShowFromHdr);
        CheckDlgButton (hDlg, IDD_SHOW_ORG_HDR, ShowOrgHdr);
        CheckDlgButton (hDlg, IDD_SHOW_REPLYTO_HDR, ShowReplyToHdr);
        CheckDlgButton (hDlg, IDD_SHOW_KEYWORDS_HDR, ShowKeywordsHdr);
        CheckDlgButton (hDlg, IDD_SHOW_SUMMARY_HDR, ShowSummaryHdr);
        CheckDlgButton (hDlg, IDD_SHOW_DISTRIBUTION_HDR, ShowDistributionHdr);
        CheckDlgButton (hDlg, IDD_SHOW_FOLLOWUPTO_HDR, ShowFollowupToHdr);
        CheckDlgButton (hDlg, IDD_ENABLE_SIG, EnableSig);
        CheckDlgButton (hDlg, IDD_X_NO_ARCHIVE, XNoArchive);
		CheckDlgButton (hDlg, IDC_STRIP_SIG, TruncateSignatureFollowup);
        SetDlgItemText (hDlg, IDD_FILENAME, SigFileName);
        SetDlgItemText (hDlg, IDD_FOLLOWUP_TEMPLATE, FollowupSaysTemplate);
        SetDlgItemText (hDlg, IDD_REPLY_TEMPLATE, ReplySaysTemplate);
		SetDlgItemText (hDlg, ID_CONFIG_TOKEN, IDToken);
		Edit_LimitText(GetDlgItem(hDlg, IDD_FILENAME), MAXFILENAME);
		Edit_LimitText(GetDlgItem(hDlg, IDD_REPLY_TEMPLATE), MAXINTERNALLINE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_TOKEN), MAXINTERNALLINE);

		SetDlgItemText (hDlg, IDC_EB_CANCELSECRET, CancelKey);
		Edit_LimitText(GetDlgItem(hDlg, IDC_EB_CANCELSECRET), MAXINTERNALLINE);
		CheckDlgButton (hDlg, IDC_USE_CANCELLOCK, UseCancelKey ? 1 : 0);

        MyShowFrom = ShowFromHdr;
        MyShowOrg = ShowOrgHdr;
        MyShowReplyTo = ShowReplyToHdr;
        MyShowKeywords = ShowKeywordsHdr;
        MyShowSummary = ShowSummaryHdr;
        MyShowDist = ShowDistributionHdr;
        MyShowFollowupTo = ShowFollowupToHdr;
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD) 409); //IDD_WINVNCOMPOSEPREFS
		break;
  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
#ifdef RFC1522
          EncodeRFC1522 = (IsDlgButtonChecked (hDlg, IDD_ENCODE_RFC1522) != 0);
#endif   
          WordWrap = (IsDlgButtonChecked (hDlg, IDD_WORD_WRAP) != 0);
          PrefillCcAddress = (IsDlgButtonChecked (hDlg, IDD_PREFILL_CC) != 0);
          ShowFromHdr = (IsDlgButtonChecked (hDlg, IDD_SHOW_FROM_HDR) != 0);
          ShowOrgHdr = (IsDlgButtonChecked (hDlg, IDD_SHOW_ORG_HDR) != 0);
          ShowReplyToHdr = (IsDlgButtonChecked (hDlg, IDD_SHOW_REPLYTO_HDR) != 0);
          ShowKeywordsHdr = (IsDlgButtonChecked (hDlg, IDD_SHOW_KEYWORDS_HDR) != 0);
          ShowSummaryHdr = (IsDlgButtonChecked (hDlg, IDD_SHOW_SUMMARY_HDR) != 0);
          ShowDistributionHdr = (IsDlgButtonChecked (hDlg, IDD_SHOW_DISTRIBUTION_HDR) != 0);
          ShowFollowupToHdr = (IsDlgButtonChecked (hDlg, IDD_SHOW_FOLLOWUPTO_HDR) != 0);
          XNoArchive = (IsDlgButtonChecked (hDlg, IDD_X_NO_ARCHIVE) != 0);
		  TruncateSignatureFollowup = (IsDlgButtonChecked (hDlg, IDC_STRIP_SIG ) != 0);
          GetDlgItemText (hDlg, IDD_FOLLOWUP_TEMPLATE, FollowupSaysTemplate, MAXINTERNALLINE - 1);
          GetDlgItemText (hDlg, IDD_REPLY_TEMPLATE, ReplySaysTemplate, MAXINTERNALLINE - 1);
          GetDlgItemText (hDlg, ID_CONFIG_TOKEN, IDToken, MAXINTERNALLINE - 1);
		  {
			IDToken[MAXINTERNALLINE - 1] = '\0';
			for (int i = 0; i < strlen(IDToken); i++)
			{
				if (isspace(IDToken[i]) || IDToken[i] == '<' || IDToken[i] == '>' || IDToken[i] == '@' || !isprint(IDToken[i]))
					IDToken[i] = '_';
			}
		  }
		  GetDlgItemText (hDlg, IDC_EB_CANCELSECRET, CancelKey, MAXINTERNALLINE - 1);
		  UseCancelKey = IsDlgButtonChecked (hDlg, IDC_USE_CANCELLOCK);
		  if (UseCancelKey && !*CancelKey)
		  {
                 WVMessageBox (hDlg, IDS_WVCNCM_INVALID_KEY, IDS_WVCNCM_KEY_TITLE, MB_OK | MB_ICONSTOP);
				 UseCancelKey = FALSE;

		  }

          if (MyShowFrom != ShowFromHdr ||
                  MyShowOrg != ShowOrgHdr ||
                  MyShowReplyTo != ShowReplyToHdr ||
                  MyShowKeywords != ShowKeywordsHdr ||
                  MyShowSummary != ShowSummaryHdr ||
                  MyShowFollowupTo != ShowFollowupToHdr ||
                  MyShowDist != ShowDistributionHdr) {
                RefreshComposeWnds ();
          }

          EnableSig = IsDlgButtonChecked (hDlg, IDD_ENABLE_SIG);
          GetDlgItemText (hDlg, IDD_FILENAME, temp, MAXFILENAME - 1);
          if (EnableSig) {
                if (*temp == '\0' || _access (temp, 0) < 0) {
                  WVMessageBox (hDlg, IDS_WVCNCM_DISABLE_SIGNATURE, IDS_WVCNCM_INVALID_FILENAME, MB_OK | MB_ICONSTOP);
                  EnableSig = FALSE;
                  break;
                }
                ResetTextBlock (Signature);
                if (ReadFileToTextBlock (hDlg, Signature, temp) == FAIL)
                  break;

                if (Signature->numLines > BIG_SIG_FILE) {
                  sprintf (str, "%lu", Signature->numLines);
                  if (WVArgMessageBox (hDlg, IDS_WVCNCM_SIGNATURE_LINES, IDS_WVCNCM_BIG_SIGNATURE,
                                  MB_YESNO | MB_ICONQUESTION, str) == IDNO) {
                        ResetTextBlock (Signature);
                        break;
                  }
                }
          }
          strcpy (SigFileName, temp);

          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case ID_BROWSE:
          if (AskForExistingFileName (hDlg, temp, "Select Signature File") == SUCCESS) {
                SetDlgItemText (hDlg, IDD_FILENAME, temp);
          }
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;

}

/*--- Function WinVnConfigArticleDlg ---------------------------------------------
 *
 * Article prefs
 *
 * jsc 9/28/94
 */
BOOL CALLBACK WinVnConfigArticleDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  int ok, num;
  static int MyItalicizeQuotes;

  switch (iMessage) {
  case WM_INITDIALOG:
		CenterWindow (hDlg, NULL );
        CheckDlgButton (hDlg, ID_WRAP_INCOMING_TEXT, WrapIncomingArticleText);
        SetDlgItemInt (hDlg, ID_WRAP_INCOMING_TEXT_LIMIT, WrapIncomingArticleTextLength, TRUE);
        CheckDlgButton (hDlg, ID_TRIM_HEADERS, TrimHeaders);
#ifdef RFC1522
        CheckDlgButton (hDlg, ID_USE_RFC1522, UseRFC1522);
#else
        EnableWindow(GetDlgItem (hDlg, ID_USE_RFC1522), FALSE);          
#endif
#ifdef VISUAL1521
        CheckDlgButton (hDlg, ID_USE_VISUAL1521, UseVISUAL1521);
#else
        EnableWindow(GetDlgItem (hDlg, ID_USE_VISUAL1521), FALSE);          
#endif
        CheckDlgButton (hDlg, ID_TRUNCATE_SIGNATURES, TruncateSignatures);
        CheckDlgButton (hDlg, ID_CONFIG_NEW_WND_ARTICLE, NewArticleWindow);
        CheckDlgButton (hDlg, ID_SCROLL_PAST_HEADERS, ScrollPastHeaders);
        CheckDlgButton (hDlg, ID_SEARCH_UNREADGROUP, ConfigJumpGroup);
/* Ray Dunn:99/10 */
        CheckDlgButton (hDlg, ID_SHOW_AUTHOR_NAME, ShowAuthorName);
/* Ray Dunn:99/10:End */
		CheckDlgButton (hDlg, ID_SHOW_XFACE, ShowXFace);
#ifndef _WIN32
		EnableWindow(GetDlgItem (hDlg, ID_SHOW_XFACE), FALSE);
#endif
        CheckDlgButton (hDlg, ID_QUOTES_IN_ITALICS, ItalicizeQuotes);
        MyItalicizeQuotes = ItalicizeQuotes;
		CheckRadioButton (hDlg, IDC_RB_TOOLBAR_SM, IDC_RB_TOOLBAR_LG, IDC_RB_TOOLBAR_SM+nToolbarArt);
        break;

  case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD) 408); //IDD_WINVNARTICLEPREFS
		break;
  
  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
          WrapIncomingArticleText = (IsDlgButtonChecked (hDlg, ID_WRAP_INCOMING_TEXT) != 0);
          num = GetDlgItemInt (hDlg, ID_WRAP_INCOMING_TEXT_LIMIT, &ok, TRUE);
          if (num && (num < 15 || num > 150)) {
                WVMessageBox (hDlg, IDS_WVCNCM_WRAP_LEN_HELP, IDS_WVCNCM_INVALID_WRAP, MB_OK | MB_ICONSTOP);
                break;
          }
          WrapIncomingArticleTextLength = num;
          if (!WrapIncomingArticleTextLength) {
                WrapIncomingArticleText = FALSE;
          }
          TrimHeaders = (IsDlgButtonChecked (hDlg, ID_TRIM_HEADERS) != 0);
#ifdef RFC1522
          UseRFC1522 = (IsDlgButtonChecked (hDlg, ID_USE_RFC1522) != 0);
#endif
#ifdef VISUAL1521
          UseVISUAL1521 = (IsDlgButtonChecked (hDlg, ID_USE_VISUAL1521) != 0);
#endif
          TruncateSignatures = (IsDlgButtonChecked (hDlg, ID_TRUNCATE_SIGNATURES) != 0);
          CheckDlgButton (hDlg, ID_SEARCH_UNREADGROUP, ConfigJumpGroup);
          NewArticleWindow = (IsDlgButtonChecked (hDlg, ID_CONFIG_NEW_WND_ARTICLE) != 0);
          ScrollPastHeaders = (IsDlgButtonChecked (hDlg, ID_SCROLL_PAST_HEADERS) != 0);
          ItalicizeQuotes = (IsDlgButtonChecked (hDlg, ID_QUOTES_IN_ITALICS) != 0);
          if (ItalicizeQuotes != MyItalicizeQuotes)
                RefreshArticleWnds ();
/* Ray Dunn:99/10 */
          ShowAuthorName = (IsDlgButtonChecked (hDlg, ID_SHOW_AUTHOR_NAME) != 0);
          /* will only change when next article is read (there's no quick way to have
             it redisplayed */
/* Ray Dunn:99/10:End */
		  ShowXFace = (IsDlgButtonChecked (hDlg, ID_SHOW_XFACE) != 0);
		  {
		    int nTemp = nToolbarArt;

			nToolbarArt = IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_SM) ? 0 :
					    IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_MD) ? 1 : 
					    IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_LG) ? 2 : 0;


			if (nTemp != nToolbarArt)
			{
				int j;

				for (j = MAXARTICLEWNDS - 1; j >= 0; j--) 
				{
				  if (ArticleDocs[j].InUse) 
				  {
					
					PostMessage( ArticleDocs[j].hWndFrame, WM_RETHINKTOOLBAR, 0, 0);
				  }
				}
				
			}
		  }

          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}


/*--- Function WinVnConfigArticleListDlg ---------------------------------------------
 *
 * Article-list (for a group) prefs
 *
 * jsc 9/28/94
 */
BOOL CALLBACK WinVnConfigArticleListDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  char numStr[20];
  static int MyFullSubject;
  static char MyDepth;
/* Ray Dunn:99/10 */
  static int MySortWithAlphaOnly;
/* Ray Dunn:99/10:End */
  static WORD MyDefaultSort;
  static HMENU MySortMenu;

  switch (iMessage) {
  case WM_INITDIALOG:
		CenterWindow (hDlg, NULL );
        SetDlgItemText (hDlg, IDD_ART_THRESHOLD, ltoa (article_threshold, numStr, 10));
        SetDlgItemText (hDlg, IDD_MIN_TO_READ, ltoa (min_to_retrieve, numStr, 10));
        CheckDlgButton (hDlg, ID_CONFIG_SHOWUNREADONLY, ShowUnreadOnly);
        CheckDlgButton (hDlg, ID_CONFIG_NEW_WND_GROUP, ViewNew);
        CheckDlgButton (hDlg, ID_CONFIG_MULTI_SELECT, ArtListMultiSelect);
        CheckDlgButton (hDlg, ID_CONFIG_SAVE_NEWSRC_ON_CLOSE, SaveNewsrcOnClose);
        CheckDlgButton (hDlg, ID_CONFIG_CROSSPOST, TrackSubscribedCrossposts);
        CheckDlgButton (hDlg, ID_CONFIG_CROSSPOST_UNSUB, TrackUnsubscribedCrossposts);
        CheckDlgButton (hDlg, ID_CONFIG_FULLNAMEFROM, FullNameFrom);
        CheckDlgButton (hDlg, ID_CONFIG_THREADS, threadp);
        CheckDlgButton (hDlg, ID_CONFIG_THREADS_FULL_SUBJECT, ThreadFullSubject);
        CheckDlgButton (hDlg, ID_ENABLE_KILLFILE, bEnableArticleAction);
/* Ray Dunn:99/10 */
        CheckDlgButton (hDlg, ID_SORT_SUBJECT_PURE_ALPHA, SortWithAlphaOnly);
/* Ray Dunn:99/10:End */
		CheckDlgButton (hDlg, IDC_CB_RIGHTMOUSE, RMDeselectOnMark);

		CheckRadioButton (hDlg, IDC_RB_TOOLBAR_SM, IDC_RB_TOOLBAR_LG, IDC_RB_TOOLBAR_SM+nToolbarView);

        SendDlgItemMessage (hDlg, ID_THREAD_DEPTH_INDICATOR, EM_LIMITTEXT, (WPARAM) 1, 0L);
        SetDlgItemText (hDlg, ID_THREAD_DEPTH_INDICATOR, ThreadDepthIndicator);

        MyFullSubject = ThreadFullSubject;
        MyDepth = *ThreadDepthIndicator;
/* Ray Dunn:99/10 */
        MySortWithAlphaOnly = SortWithAlphaOnly;
/* Ray Dunn:99/10:End */
        MyDefaultSort = ReadSortOption("","");
        MySortMenu = NULL;
        break;

  case WM_INITMENU:
    if (MySortMenu)
    {
      UncheckAllSortMenuItems(MySortMenu);
      CheckSortMenuItem(MySortMenu, MyDefaultSort, MF_CHECKED);
    }

    break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD) 407); //IDD_WINVNARTICLELISTPREFS
		break;
  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDOK:
          ShowUnreadOnly = (IsDlgButtonChecked (hDlg, ID_CONFIG_SHOWUNREADONLY) != 0);
          ViewNew = (IsDlgButtonChecked (hDlg, ID_CONFIG_NEW_WND_GROUP) != 0);
          ArtListMultiSelect = (IsDlgButtonChecked (hDlg, ID_CONFIG_MULTI_SELECT) != 0);
          SaveNewsrcOnClose = (IsDlgButtonChecked (hDlg, ID_CONFIG_SAVE_NEWSRC_ON_CLOSE) != 0);
          TrackSubscribedCrossposts = (IsDlgButtonChecked (hDlg, ID_CONFIG_CROSSPOST) != 0);
          TrackUnsubscribedCrossposts = (IsDlgButtonChecked (hDlg, ID_CONFIG_CROSSPOST_UNSUB) != 0);
          FullNameFrom = (IsDlgButtonChecked (hDlg, ID_CONFIG_FULLNAMEFROM) != 0);
          threadp = (IsDlgButtonChecked (hDlg, ID_CONFIG_THREADS) != 0);
          bEnableArticleAction = (IsDlgButtonChecked (hDlg, ID_ENABLE_KILLFILE) != 0);
          ThreadFullSubject = (IsDlgButtonChecked (hDlg, ID_CONFIG_THREADS_FULL_SUBJECT) != 0);
          GetDlgItemText (hDlg, ID_THREAD_DEPTH_INDICATOR, ThreadDepthIndicator, 2);
          ThreadDepthIndicator[1] = '\0';
/* Ray Dunn:99/10 */
          SortWithAlphaOnly = (IsDlgButtonChecked (hDlg, ID_SORT_SUBJECT_PURE_ALPHA) != 0);
          if (MyDefaultSort)
          {
            WriteDefaultSortOption(MyDefaultSort);
          }
          if (ThreadFullSubject != MyFullSubject || *ThreadDepthIndicator != MyDepth ||
              MySortWithAlphaOnly != SortWithAlphaOnly)
          {
                RefreshGroupWnds (TRUE);
          }
		  RMDeselectOnMark = (IsDlgButtonChecked( hDlg, IDC_CB_RIGHTMOUSE ) != 0);

/* Ray Dunn:99/10:End */
		  {
		    int nTemp = nToolbarView;

			nToolbarView = IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_SM) ? 0 :
					    IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_MD) ? 1 : 
					    IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_LG) ? 2 : 0;

			if (nTemp != nToolbarView)
			{
				int j;

				for (j = MAXGROUPWNDS - 1; j >= 0; j--) 
				{
				  if (GroupDocs[j].InUse) 
				  {
					
					PostMessage( GroupDocs[j].hWndFrame, WM_RETHINKTOOLBAR, 0, 0);
				  }
				}
				
			}
		  }
          GetDlgItemText (hDlg, IDD_ART_THRESHOLD, numStr, 20);
          if (atol(numStr) <= 0){
             EndDialog(hDlg, FALSE);
          }
          else 
		  {
             article_threshold = atol (numStr);
             GetDlgItemText (hDlg, IDD_MIN_TO_READ, numStr, 20);
             min_to_retrieve = atol (numStr);

            /* 0 or a bogus value means they don't want a threshold */
            if (!article_threshold)
                article_threshold = INT_MAX;
			EndDialog (hDlg, TRUE);
          }
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case ID_SET_DEFAULT_SORT:
          {
            
            HMENU hSortMenu;
            MySortMenu = LoadMenu(hInst, "VIEWMENU");
            if (MySortMenu)
            {
              hSortMenu = GetSubMenu(MySortMenu, 1);
              if (hSortMenu)
              {
                POINT pt;
                GetCursorPos(&pt);
                //replace "filters" menu with something
                DeleteMenu(hSortMenu, 0, MF_BYPOSITION);
                InsertMenu(hSortMenu, 0, MF_BYPOSITION, -1, "Choose Default Sorting");
                //but disable it
                EnableMenuItem(hSortMenu, 0, MF_BYPOSITION | MF_GRAYED);
                TrackPopupMenu(hSortMenu, TPM_LEFTALIGN, pt.x, pt.y, 0, hDlg, NULL);
                DestroyMenu(MySortMenu);
                MySortMenu = NULL;
              }
            }
          }

          break;
        
        case IDM_DONT_SORT:
        case IDM_SORT_DATE_TIME:
        case IDM_SORT_DATE_SUBJECT:
        case IDM_SORT_DATE_LINES:
        case IDM_SORT_DATE_ARTNUM:
        case IDM_SORT_DATE_FROM:
        case IDM_SORT_SUBJECT_DATE:
        case IDM_SORT_SUBJECT_LINES:
        case IDM_SORT_SUBJECT_ARTNUM:
        case IDM_SORT_SUBJECT_FROM:
        case IDM_SORT_THREADS:
        case IDM_SORT_THREADSUB:
        case IDM_SORT_LINES:
        case IDM_SORT_ARTNUM:
        case IDM_SORT_FROM_SUBJECT:
        case IDM_SORT_FROM_DATE:
        case IDM_SORT_FROM_LINES:
        case IDM_SORT_FROM_ARTNUM:
		case IDM_SORT_FILENAME:
          MyDefaultSort = LOWORD(wParam);
          break;
        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*--- Function WinVnConfigGroupListDlg ---------------------------------------------
 *
 * Group-list (the usenet window) prefs
 *
 * jsc 9/28/94
 */
BOOL CALLBACK WinVnConfigGroupListDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  int temp, item;

  switch (iMessage) {
  case WM_INITDIALOG:
		CenterWindow (hDlg, NULL );
        CheckRadioButton (hDlg, ID_DOLIST_BASE, ID_DOLIST_ASK, DoList + ID_DOLIST_BASE);
        CheckDlgButton (hDlg, ID_CONFIG_MULTI_SELECT, GroupListMultiSelect);
        CheckDlgButton (hDlg, ID_CONFIG_SHOWUNSUB, ShowUnsubscribed);
        CheckDlgButton (hDlg, ID_CONFIG_SORT_GROUPLIST, SortGroupList);
		CheckRadioButton (hDlg, IDC_RB_TOOLBAR_SM, IDC_RB_TOOLBAR_LG, IDC_RB_TOOLBAR_SM+nToolbarMain);
        break;

  case WM_HELP:
	DoHelp (hDlg, HELP_CONTEXT, (DWORD) 406); //IDD_WINVNGROUPLIST
	break;
  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {
        case IDOK:
          GroupListMultiSelect = (IsDlgButtonChecked (hDlg, ID_CONFIG_MULTI_SELECT) != 0);

          for (item = ID_DOLIST_BASE; item <= ID_DOLIST_ASK; item++) {
                if (IsDlgButtonChecked (hDlg, item)) {
                  DoList = item - ID_DOLIST_BASE;
                }
          }
          /* If the user has changed the ShowUnsubscribed option, we must
           * recompute which lines are active and redisplay.  
           * Reset the window to the top to make sure that everything
           * will display OK.
           * There is some question in my mind whether this code will
           * work OK if the first screen's worth of groups are unsubscribed,
           * but it *should* work.
           */
          temp = (IsDlgButtonChecked (hDlg, ID_CONFIG_SHOWUNSUB) != 0);
          if (temp != ShowUnsubscribed) {
                ShowUnsubscribed = temp;
                NetDoc.LongestLine = 0;
                SetGroupActiveLines ();
                ScreenToTop (&NetDoc);
                InvalidateRect (NetDoc.hDocWnd, NULL, FALSE);
                SendMessage (NetDoc.hWndTB, TB_CHECKBUTTON, IDB_TOGGLE_VIEW_UNSUB, ShowUnsubscribed);
          }
          SortGroupList = IsDlgButtonChecked (hDlg, ID_CONFIG_SORT_GROUPLIST);
		  { int nTemp = nToolbarMain;
			nToolbarMain = IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_SM) ? 0 :
					    IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_MD) ? 1 : 
					    IsDlgButtonChecked (hDlg, IDC_RB_TOOLBAR_LG) ? 2 : 0;

			if (nTemp != nToolbarMain)
				PostMessage( GetParent(hDlg), WM_RETHINKTOOLBAR, 0 , 0);
		  }
		  EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*--- Function WinVnConfigCodingPrefsDlg ---------------------------------------------
 *
 * Decoding prefs
 *
 * jsc 9/28/94
 */
BOOL CALLBACK WinVnConfigCodingPrefsDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  static int MyAlwaysOnTop;
  int nSel;
  char szEntry[_MAX_PATH];

  switch (iMessage) {
  case WM_INITDIALOG:
        {
          char szExtension[256];
          char *this_hop,*next_hop;
          HWND hListBox;

          CheckDlgButton (hDlg, ID_STATUSALWAYSONTOP, BlockCodingStatusAlwaysOnTop);
          CheckDlgButton (hDlg, IDD_VERBOSE_STATUS, CodingStatusVerbose);
          CheckDlgButton (hDlg, IDD_MINIMIZE_CODING_STATUS, MinimizeStatusWindows);
          CheckDlgButton (hDlg, IDD_KEEP_HEADER_VISIBLE, KeepArticleHeaderVisible);
          CheckDlgButton (hDlg, IDD_EXECUTE_DECODED, ExecuteDecodedFiles);
          CheckDlgButton (hDlg, IDD_DUMB_DECODE, DumbDecode);
          CheckDlgButton (hDlg, IDD_INCLUDE_OPEN_ARTS, AlsoDecodeOpenArticles);
		  CheckDlgButton (hDlg, IDC_CB_DECODE_NOEXE, NeverExeWarnFiles);
          MyAlwaysOnTop = BlockCodingStatusAlwaysOnTop;
          EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_ADD), FALSE);
          EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_DEL), FALSE);
          Edit_LimitText(GetDlgItem (hDlg, IDC_EB_DECODE_EDITWARN), 12);

		  CheckDlgButton(hDlg, IDC_CB_PLAYSOUND, EnableDownloadSnd);
		  if (!EnableDownloadSnd)
		  {
			EnableWindow(GetDlgItem (hDlg, IDC_PB_CHOOSESND), FALSE);
			EnableWindow(GetDlgItem (hDlg, IDC_EB_SOUND), FALSE);
			
		  }
		  SetDlgItemText (hDlg, IDC_EB_SOUND, szDownloadSnd);

          GetPrivateProfileString(CODING, WARNEXTENTIONS, DEF_WarnOnEXE, 
                                  szExtension, 256, szAppProFile);
          
          hListBox = GetDlgItem (hDlg, IDC_LB_DECODE_WARNLIST);
          if (hListBox)
          {
            next_hop = this_hop = szExtension;
            while (this_hop && *this_hop)
            {
              next_hop = strchr (this_hop, ',');
              if (next_hop)
                *(next_hop++) = '\0';
              
              while( isspace(*this_hop))
                this_hop++;

              ListBox_AddString(hListBox, this_hop);
              this_hop = next_hop;

            }
          }
        }
        break;

  case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD) 411); //IDD_WINVNCODINGPREFS
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {

        case IDOK:
          {
            int nCount;
            HWND hListbox = GetDlgItem (hDlg, IDC_LB_DECODE_WARNLIST);
            char szExtension[256];
            *szExtension = '\0';
             
            BlockCodingStatusAlwaysOnTop = (IsDlgButtonChecked (hDlg, ID_STATUSALWAYSONTOP) != 0);
            CodingStatusVerbose = (IsDlgButtonChecked (hDlg, IDD_VERBOSE_STATUS) != 0);
            MinimizeStatusWindows = (IsDlgButtonChecked (hDlg, IDD_MINIMIZE_CODING_STATUS) != 0);
            KeepArticleHeaderVisible = (IsDlgButtonChecked (hDlg, IDD_KEEP_HEADER_VISIBLE) != 0);
            ExecuteDecodedFiles = (IsDlgButtonChecked (hDlg, IDD_EXECUTE_DECODED) != 0);
            DumbDecode = IsDlgButtonChecked (hDlg, IDD_DUMB_DECODE);
            AlsoDecodeOpenArticles = (IsDlgButtonChecked (hDlg, IDD_INCLUDE_OPEN_ARTS) != 0);
            NeverExeWarnFiles = (IsDlgButtonChecked (hDlg, IDC_CB_DECODE_NOEXE ) != 0);

            nCount = ListBox_GetCount(hListbox);
            nSel = 0;
            while (nCount-- > 0)
            {
              int nLength;
              nLength = ListBox_GetText(hListbox, nSel++, szEntry);
              if (nLength > 0 && (strlen( szExtension) + nLength  < 256))
              { 
                strcat(szExtension, szEntry);
                strcat(szExtension, ",");
              }
              
            }
            if (nSel)
              szExtension[strlen( szExtension)-1] = '\0';
            WritePrivateProfileString (CODING, WARNEXTENTIONS, szExtension, szAppProFile);
            
            if (BlockCodingStatusAlwaysOnTop != MyAlwaysOnTop)
                  RefreshCodedBlockWnd ();

			if (IsDlgButtonChecked (hDlg, IDC_CB_PLAYSOUND))
			{
				EnableDownloadSnd = 1;
				*szEntry = '\0';
				GetDlgItemText (hDlg, IDC_EB_SOUND, szEntry, 255);
				
				if (*szEntry && (_access( szEntry, 0 ) == 0 || strchr(szEntry, '%')))
					strcpy(szDownloadSnd, szEntry);
			}
			else
			{
				EnableDownloadSnd = 0;
			}


            EndDialog (hDlg, TRUE);
          }
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;
		case IDC_CB_PLAYSOUND:
		  {
			BOOL bEnable = IsDlgButtonChecked (hDlg, IDC_CB_PLAYSOUND);
			EnableWindow(GetDlgItem (hDlg, IDC_PB_CHOOSESND), bEnable);
			EnableWindow(GetDlgItem (hDlg, IDC_EB_SOUND), bEnable);
			if (bEnable)
			{
				*szEntry = '\0';
				GetDlgItemText (hDlg, IDC_EB_SOUND, szEntry, 255);
				if (*szEntry && (_access( szEntry, 0 ) == 0 || strchr(szEntry, '%')) )
					break;

			}
			else
			{
				break;
			}
		  }
		  //fall through
		case IDC_PB_CHOOSESND:
		{
		  OPENFILENAME ofn;
		  char szFile[_MAX_FNAME];
		  char szPath[_MAX_PATH];
		  char szDrive[_MAX_DRIVE];
		  char szDir[_MAX_DIR];
		  char szExtn[_MAX_EXT];
		  
		  
		  szFile[0] = '\0';
		  szPath[0] = '\0';
		  GetDlgItemText (hDlg, IDC_EB_SOUND, szEntry, 255);
		  if (*szEntry)
		  {
			_splitpath( szEntry, szDrive, szDir, szFile, szExtn);
			_makepath( szPath, szDrive, szDir, NULL, NULL);
			strcat(szFile, szExtn);
		  }

		  memset (&ofn, 0, sizeof (OPENFILENAME));
		  ofn.lStructSize = sizeof(OPENFILENAME);
		  ofn.lpstrTitle = "Choose Sound";
		  ofn.lStructSize = sizeof (OPENFILENAME);
		  ofn.hwndOwner = hDlg;
		  ofn.lpstrFilter = "All Files (*.*)\0*.*\0Sound files (*.WAV)\0*.wav\0\0";
		  ofn.nFilterIndex = 2;
		  ofn.lpstrFile = szFile;
		  ofn.nMaxFile = _MAX_PATH;
		  ofn.lpstrInitialDir = szPath;
#ifdef _WIN32
		  ofn.Flags = OFN_FILEMUSTEXIST | OFN_EXPLORER | OFN_HIDEREADONLY;
#else
		  ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
#endif

		  if (GetOpenFileName (&ofn) == 0)
			return FALSE;
		  SetDlgItemText (hDlg, IDC_EB_SOUND, szFile);
 
		}
		break;


        case IDC_LB_DECODE_WARNLIST:
          {
            if (GET_WM_COMMAND_CMD(wParam, lParam) == LBN_SELCHANGE)
            {

              HWND hListbox = GET_WM_COMMAND_HWND(wParam, lParam);
              nSel = ListBox_GetCurSel(hListbox);
              if (nSel != LB_ERR)
              {
                ListBox_GetText(hListbox, nSel, szEntry);
                SetDlgItemText(hDlg, IDC_EB_DECODE_EDITWARN, szEntry);
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_ADD), FALSE);
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_DEL), TRUE);
              }
            }
          }
          break;
        case IDC_PB_DECODE_DEL:
          {
            HWND hListbox = GetDlgItem (hDlg, IDC_LB_DECODE_WARNLIST);
            nSel = ListBox_GetCurSel(hListbox);
            if (nSel != LB_ERR)
            {
              ListBox_DeleteString(hListbox, nSel);
              if (nSel > 0)
                nSel--;
              if (ListBox_GetCount(hListbox))
              {
                ListBox_SetCurSel(hListbox, nSel);
                ListBox_GetText(hListbox, nSel, szEntry);
                SetDlgItemText(hDlg, IDC_EB_DECODE_EDITWARN, szEntry);
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_ADD), FALSE);
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_DEL), TRUE);
              }
              else
              {
                SetDlgItemText(hDlg, IDC_EB_DECODE_EDITWARN, "");
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_ADD), FALSE);
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_DEL), FALSE);

              }
            }
            
          }
          break;
        case IDC_PB_DECODE_ADD:
          {
            char *p;

            HWND hListbox = GetDlgItem (hDlg, IDC_LB_DECODE_WARNLIST);
            HWND hEdit = GetDlgItem (hDlg, IDC_EB_DECODE_EDITWARN);
            Edit_GetText(hEdit, szEntry, 255);
            RemoveTrailingWhiteSpace(szEntry);
            p = szEntry;
            while (isspace(*p) || *p == '.')
              p++;

            nSel = ListBox_FindStringExact(hListbox, -1, szEntry);
            if (nSel == LB_ERR)
            {
              nSel = ListBox_AddString(hListbox, p);
              ListBox_SetCurSel(hListbox, nSel);
              SetDlgItemText(hDlg, IDC_EB_DECODE_EDITWARN, "");
            }
            else
            {
              MessageBeep((UINT) -1);
              SetDlgItemText(hDlg, IDC_EB_DECODE_EDITWARN, szEntry);  
              ListBox_SetCurSel(hListbox, nSel);
            }
          }
          break;
        case IDC_PB_DECODE_RESET:
          {
            char *this_hop, *next_hop;
            HWND hListBox = GetDlgItem (hDlg, IDC_LB_DECODE_WARNLIST);
            if (WVMessageBox(hDlg, IDS_WVCNCM_CONFIRM_RESET, IDS_WINVN_CONFIRM1, MB_ICONQUESTION | MB_YESNO) == IDNO)
              break;
            if (hListBox)
            {
              strcpy(szEntry, DEF_WarnOnEXE);
              ListBox_ResetContent(hListBox);
              next_hop = this_hop = szEntry;
              while (this_hop && *this_hop)
              {
                next_hop = strchr (this_hop, ',');
                if (next_hop)
                  *(next_hop++) = '\0';
              
                while( isspace(*this_hop))
                  this_hop++;

                ListBox_AddString(hListBox, this_hop);
                this_hop = next_hop;

              }
              SetDlgItemText(hDlg, IDC_EB_DECODE_EDITWARN, "");
              EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_ADD), FALSE);
              EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_DEL), FALSE);
            }
          }
          break;
        case IDC_EB_DECODE_EDITWARN:
          {
            
            if (GET_WM_COMMAND_CMD(wParam, lParam) == EN_CHANGE)
            {
             
              HWND hListbox = GetDlgItem (hDlg, IDC_LB_DECODE_WARNLIST);
              HWND hEdit = GET_WM_COMMAND_HWND(wParam, lParam);
              Edit_GetText(hEdit, szEntry, 255);
              nSel = ListBox_FindStringExact(hListbox, -1, szEntry);
              if (nSel == LB_ERR)
              {
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_ADD), TRUE);
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_DEL), FALSE);
              }
              else
              {
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_ADD), FALSE);
                EnableWindow(GetDlgItem (hDlg, IDC_PB_DECODE_DEL), TRUE);
                ListBox_SetCurSel(hListbox, nSel);
              }
            }

          }
          break;
		case IDC_CB_DECODE_NOEXE:
          {
		  }
		  break;
	
        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*--- Function WinVnConfigConfirmationDlg ---------------------------------------------
 *
 * Confirmations prefs
 *
 * jsc 9/28/94
 */
BOOL CALLBACK WinVnConfigConfirmationDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	HWND hCtrl;
#ifdef OFFLINE  // TANAKA Goh
  char* pGenSockDLL;
#endif

  switch (iMessage) {
  case WM_INITDIALOG:
		hCtrl = GetDlgItem (hDlg, ID_CONFIG_CONFIRM_DELETE_ARTICLE);
        CheckDlgButton (hDlg, ID_CONFIG_CONFIRM_BATCH, ConfirmBatchOps);
        CheckDlgButton (hDlg, ID_CONFIRM_DISCONNECT, ConfirmDisconnect);
        CheckDlgButton (hDlg, ID_CONFIG_CONFIRM_REPLY_TO, ConfirmReplyTo);
        CheckDlgButton (hDlg, ID_CONFIG_CONFIRM_EXIT, ConfirmSaveOnExit);
		CheckDlgButton(hDlg, ID_CONFIRM_EXITACTIVE, ConfirmExitActive);
/* Ray Dunn:99/10 */
        CheckDlgButton (hDlg, ID_CONFIG_CONFIRM_XPOSTING, ConfirmXPosting);
/* Ray Dunn:99/10:End */

#ifdef OFFLINE  // TANAKA Goh
        CheckDlgButton (hDlg, ID_CONFIG_CONFIRM_DELETE_ARTICLE, ConfirmDeleteArticle);
        
        if ((pGenSockDLL = strrchr(GenSockDLL, '\\')) == NULL) 
		{
          pGenSockDLL = GenSockDLL;
        } 
		else 
		{
          pGenSockDLL++;
        }
        
        if (stricmp(pGenSockDLL, "gwssoup.dll") && stricmp(pGenSockDLL, "gsoup32.dll") && stricmp(pGenSockDLL, "ggnspool.dll") && stricmp(pGenSockDLL, "gspool32.dll")) {
          EnableWindow (hCtrl, DISABLE);
        }
#else
		if (hCtrl)
		{
			EnableWindow (hCtrl, DISABLE);
			ShowWindow(hCtrl, SW_HIDE);
		}
#endif
        break;

	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD) 405); //IDD_WINVNCONFIRMATIONS
		break;


  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) 
		{


        case IDOK:
          ConfirmBatchOps = (IsDlgButtonChecked (hDlg, ID_CONFIG_CONFIRM_BATCH) != 0);
          ConfirmDisconnect = (IsDlgButtonChecked (hDlg, ID_CONFIRM_DISCONNECT) != 0);
          ConfirmReplyTo = (IsDlgButtonChecked (hDlg, ID_CONFIG_CONFIRM_REPLY_TO) != 0);
          ConfirmSaveOnExit = (IsDlgButtonChecked (hDlg, ID_CONFIG_CONFIRM_EXIT) != 0);
          ConfirmExitActive = (IsDlgButtonChecked (hDlg, ID_CONFIRM_EXITACTIVE) != 0);
/* Ray Dunn:99/10 */
          ConfirmXPosting = (IsDlgButtonChecked (hDlg, ID_CONFIG_CONFIRM_XPOSTING) != 0);
/* Ray Dunn:99/10:End */

#ifdef OFFLINE  // TANAKA Goh
          ConfirmDeleteArticle = (IsDlgButtonChecked (hDlg, ID_CONFIG_CONFIRM_DELETE_ARTICLE) != 0);
#endif
          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*-- function WinVnThresholdDlg ---------------------------------------
 *
 *  Dialog function to ask how many article headers we
 *  want to retrieve.
 */
/* Ray Dunn:99/12: Must use WPARAM & LPARAM */
#define MAX_CUSTOM_ARTS 6
BOOL CALLBACK WinVnThresholdDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  char numStr[20];
  char numreqStr[20];
  static int radio_id = 1;
  static int nMsgCount[MAX_CUSTOM_ARTS] = { 100,250,500,1000,10000,100000};

  switch (iMessage) 
  {

  case WM_INITDIALOG:
/* Ray Dunn:99/10: Add the number of articles that will be loaded for first unseen
   if different from number unseen - so if there's one unseen way back, the user knows
   that he will be loading a long list to get it
*/
        //CenterWindow(hDlg, GetParent(hDlg));
		sprintf (str, "%lu", arts_to_retrieve);
        sprintf (numStr, "%lu", est_num_unread);
        if (arts_reqd_for_first_unseen > 0 && est_num_unread != arts_reqd_for_first_unseen)
        {
          sprintf (numreqStr, "%lu", arts_reqd_for_first_unseen); 
          WVArg3SetWindowText(hDlg, IDS_WVCNCM_UNSEEN2, str, numStr, numreqStr);
        }
        else
          WVArg2SetWindowText (hDlg, IDS_WVCNCM_UNSEEN, str, numStr);
/* Ray Dunn:99/10:End */        
/* Ray Dunn:99/11: No Sort Enhancement */
        InhibitSort = FALSE;
        CheckDlgButton (hDlg, IDC_INHIBIT_SORT, InhibitSort);

        EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);
/* Ray Dunn:99/11:End */
		CheckRadioButton(hDlg, IDC_RB_LASTEST, IDC_RB_RANGE, IDC_RB_LASTEST);
		radio_id = 1;
		
		sprintf (numStr, "%lu", art_num);
		SetDlgItemText(hDlg, IDC_EB_START, numStr);

		EnableWindow(GetDlgItem(hDlg, IDC_EB_START), FALSE);
		WVArgXSetWindowText(GetDlgItem(hDlg, IDC_ST_RANGE), IDS_WVCNCM_AVAILABLE, server_first, server_last);
		{
			for (int i = 0; i < MAX_CUSTOM_ARTS; i++)
			{
				HWND hWnd = GetDlgItem(hDlg, ID_ARTS_1+i);
				wsprintf(numreqStr, "Threshold%01d", i+1);

				int nTemp = GetPrivateProfileInt (ARTLIST, numreqStr, 0, szAppProFile);
				if (nTemp > 9 )
				{
					nMsgCount[i] = nTemp;

					if (nTemp > 9999)
						nTemp = nTemp /1000;
					itoa(nTemp, numreqStr+1, 10);
					if (nMsgCount[i] > 9999)
						strcat(numreqStr, "K");
					*numreqStr = '&';
					SetWindowText(hWnd, numreqStr);
				
				}

				
				if (hWnd && nMsgCount[i] >= server_last - server_first)
					EnableWindow(hWnd, FALSE);
			}
		}
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD) IDD_THRESHOLD);
		break; 

  case WM_COMMAND:
/* Ray Dunn:99/12: Use GET_WM_COMMAND_ID */
    switch (GET_WM_COMMAND_ID(wParam, lParam)) 
	{
       
		case IDOK:
          
		  GetDlgItemText (hDlg, ID_ARTS_TO_GRAB, numStr, 20);
          arts_to_retrieve = atol (numStr);
          if (	radio_id == IDC_RB_RANGE)
		  {
			  unsigned long val;
			  GetDlgItemText (hDlg, IDC_EB_START, numStr, 20);
			  val = atol (numStr);
			  if (val < server_first || val > server_last)
			  {
			    WVMessageBox (hDlg, IDS_WVCNCM_OUT_OF_RANGE, IDS_WVCNCM_OUT_OF_RANGE_TITLE, MB_OK);	
			    if (val < server_first)
				  SetDlgItemText(hDlg, IDC_EB_START, ltoa(server_first, numStr, 10));
			   
			    break;
			  }
			  server_first = val;
		   }
		   else if (radio_id == IDC_RB_EARLIEST)
		   {

		   }

		  if (arts_to_retrieve > 0)
				EndDialog (hDlg, radio_id);
		  else
		  {
			arts_to_retrieve = 0;
			EndDialog (hDlg, TRUE); // Heirich 19991017 should work like ID_THRESHOLD_UNREAD not ID_CANCEL 
		  }


          break;

        case IDC_INHIBIT_SORT:
          InhibitSort = IsDlgButtonChecked (hDlg, IDC_INHIBIT_SORT) != 0;
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

		case ID_ARTS_1:
		case ID_ARTS_2:
		case ID_ARTS_3:
		case ID_ARTS_4:
		case ID_ARTS_5:
		case ID_ARTS_6:
		case ID_ARTS_7:
		{
			arts_to_retrieve = nMsgCount[ GET_WM_COMMAND_ID(wParam, lParam) - ID_ARTS_1];

			if (radio_id == 1)
				SetDlgItemText(hDlg, IDC_EB_START, ltoa(server_last - arts_to_retrieve, numStr, 10));
			SetDlgItemText(hDlg, ID_ARTS_TO_GRAB, ltoa(arts_to_retrieve, numStr, 10));
			PostMessage(hDlg, WM_COMMAND, (WPARAM)IDOK, (LPARAM)hDlg); 
		}
        break;

        case ID_THRESHOLD_ALL:
		  SetDlgItemText(hDlg, IDC_EB_START, ltoa(server_first, numStr, 10));
		  EndDialog (hDlg, ID_THRESHOLD_ALL);
          break;

        case ID_THRESHOLD_UNREAD:
          if (est_num_unread > 0 || min_to_retrieve > 0) //MDS don't ask for 0 articles, unless fetch min is set
            EndDialog (hDlg, ID_THRESHOLD_UNREAD);
          break;

        case ID_ARTS_TO_GRAB:
          // if user fills a number to load,
          // change the default button to the OK buton
          if (GET_WM_COMMAND_CMD(wParam, lParam) == EN_CHANGE)
          { char buf[64];

            GetDlgItemText(hDlg, ID_ARTS_TO_GRAB, buf, sizeof(buf));
            if (*buf == '\0' || strncmp(buf, "                      ", strlen(buf)) == 0)
            {
              EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);
              SendMessage(hDlg, DM_SETDEFID, (WPARAM)ID_THRESHOLD_UNREAD, (LPARAM) 0);
            }
            else
            {
				unsigned long first;
				EnableWindow(GetDlgItem(hDlg, IDOK), TRUE);
				SendMessage(hDlg, DM_SETDEFID, (WPARAM)IDOK, (LPARAM) 0);
				first = atol(buf);

				if (radio_id == 1)
				{
					
					if (first > server_last-server_first)
						first = server_first;
					else
						first = server_last - first;
					
					SetDlgItemText(hDlg, IDC_EB_START, ltoa(first, numStr, 10));
				}

				
            }
          }
          break;

		case IDC_RB_RANGE:
			EnableWindow(GetDlgItem(hDlg, IDC_EB_START), TRUE);
			radio_id = IDC_RB_RANGE; 
			break;
		case IDC_RB_LASTEST:
			radio_id = 1;
			EnableWindow(GetDlgItem(hDlg, IDC_EB_START), FALSE);
			break;
		case IDC_RB_EARLIEST:
		    radio_id = IDC_RB_EARLIEST;
			SetDlgItemText(hDlg, IDC_EB_START, ltoa(server_first, numStr, 10));
			EnableWindow(GetDlgItem(hDlg, IDC_EB_START), FALSE);
			break;
        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}


/*--- Function WinVnLogOptDlg ---------------------------------------------
 *
 * Logging prefs
 *
 * jsc 9/28/94
 */
BOOL CALLBACK WinVnLogOptDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  char fileName[MAXFILENAME];
  UINT mode;
  OFSTRUCT ofs;
  HFILE hFile;

  switch (iMessage) {

  case WM_INITDIALOG:
        CheckDlgButton (hDlg, IDD_MAILLOG, MailLog);
        CheckDlgButton (hDlg, IDC_MAIL_ATTACH, MailLogAttach);
        CheckDlgButton (hDlg, IDD_POSTLOG, PostLog);
        CheckDlgButton (hDlg, IDC_POST_ATTACH, PostLogAttach);
        SetDlgItemText (hDlg, IDD_MAILNAME, MailLogFile);
        SetDlgItemText (hDlg, IDD_POSTNAME, PostLogFile);

        /* mail logging only works for SMTP-mail at present */
        EnableWindow (GetDlgItem (hDlg, IDD_MAILLOG), (MailCtrl.MailType == MT_SMTP));
        EnableWindow (GetDlgItem (hDlg, IDC_MAIL_ATTACH), (MailCtrl.MailType == MT_SMTP));
        EnableWindow (GetDlgItem (hDlg, IDD_MAILNAME), (MailCtrl.MailType == MT_SMTP));
        EnableWindow (GetDlgItem (hDlg, ID_BROWSEMAIL), (MailCtrl.MailType == MT_SMTP));
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)403); //IDD_WINVNLOGOPTS
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {

        case IDOK:
          GetDlgItemText (hDlg, IDD_MAILNAME, fileName, MAXFILENAME - 1);
          mode = (OpenFile (fileName, &ofs, OF_EXIST) == HFILE_ERROR) ? OF_CREATE : OF_WRITE;
          if ((hFile = OpenFile (fileName, &ofs, mode)) == HFILE_ERROR) {
                WVArgMessageBox (hDlg, IDS_WVCNCM_DISABLE_MAILLOG, IDS_WVCNCM_INVALID_FILE, MB_OK | MB_ICONSTOP, fileName);
                CheckDlgButton (hDlg, IDD_MAILLOG, FALSE);
                break;
          }
          _lclose (hFile);
          strcpy (MailLogFile, fileName);

          GetDlgItemText (hDlg, IDD_POSTNAME, fileName, MAXFILENAME - 1);
          mode = (OpenFile (fileName, &ofs, OF_EXIST) == HFILE_ERROR) ? OF_CREATE : OF_WRITE;
          if ((hFile = OpenFile (fileName, &ofs, mode)) == HFILE_ERROR) {
                WVArgMessageBox (hDlg, IDS_WVCNCM_DISABLE_POSTLOG, IDS_WVCNCM_INVALID_FILE, MB_OK | MB_ICONSTOP, fileName);
                CheckDlgButton (hDlg, IDD_POSTLOG, FALSE);
                break;
          }
          _lclose (hFile);
          strcpy (PostLogFile, fileName);

          MailLog = (IsDlgButtonChecked (hDlg, IDD_MAILLOG) != 0);
          MailLogAttach = (IsDlgButtonChecked (hDlg, IDC_MAIL_ATTACH) != 0);
          PostLog = (IsDlgButtonChecked (hDlg, IDD_POSTLOG) != 0);
          PostLogAttach = (IsDlgButtonChecked (hDlg, IDC_POST_ATTACH) != 0);
          EndDialog (hDlg, TRUE);
          break;

        case IDCANCEL:
          EndDialog (hDlg, FALSE);
          break;

        case ID_BROWSEPOST:
          fileName[0] = '\0';
          if (AskForNewFileName (hDlg, fileName, "", TRUE) == SUCCESS)
                SetDlgItemText (hDlg, IDD_POSTNAME, fileName);
          break;

        case ID_BROWSEMAIL:
          fileName[0] = '\0';
          if (AskForNewFileName (hDlg, fileName, "", TRUE) == SUCCESS)
                SetDlgItemText (hDlg, IDD_MAILNAME, fileName);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

/*-- function WinVnMailDialog ---------------------------------------
 *
 *  Dialog function to query for mail destination, and manage list
 *  of mail addresses
 *
 *  Returns mail address(es) in DialogString
 *  (JSC 2/1/94)
 */

BOOL CALLBACK WinVnMailDlg (HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
  static HWND hMailList;                /* Handle to child list box window. */
  register int j;
  int count, len, startLen, found;
  char *ptr;
  char mybuf[MAXDIALOGSTRING];

  switch (iMessage) {
  case WM_INITDIALOG:
        ptr = (char *) lParam;
        if (ptr && *ptr) {
          SetDlgItemText (hDlg, IDD_MAIL_DEST, ptr);
        }

        hMailList = GetDlgItem (hDlg, IDD_MAIL_LIST);

        SendMessage (hMailList, WM_SETREDRAW, FALSE, 0L);
        for (j = 0; j < (int) MailList->numLines; j++)
          SendMessage (hMailList, LB_ADDSTRING, 0, (LPARAM) (LPSTR) TextBlockLine (MailList, j));

        SendMessage (hMailList, WM_SETREDRAW, TRUE, 0L);
        EnableWindow (GetDlgItem (hDlg, IDD_DELETE), FALSE);
        EnableWindow (GetDlgItem (hDlg, IDD_ADD), FALSE);
        return TRUE;
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNMAIL);
		break;

  case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {


        case IDD_MAIL_LIST:
          switch (GET_WM_COMMAND_CMD (wParam, lParam)) {
          case LBN_SELCHANGE:
                EnableWindow (GetDlgItem (hDlg, IDD_DELETE), TRUE);
                break;

          case LBN_DBLCLK:
                // a double click means they want to add a name to the to-line
                // only add it if it's not already on the to-line

                j = (int) SendMessage (hMailList, LB_GETCARETINDEX, 0, 0L);
                SendMessage (hMailList, LB_GETTEXT, j, (LPARAM) (LPSTR) mybuf);

                GetDlgItemText (hDlg, IDD_MAIL_DEST, DialogString, MAXDIALOGSTRING - 1);
                strcpy (str, DialogString);
                // test if name already on "To:" line
                // can't to a test with strstr() because mybuf may be a substring
                // of another address (i.e. 'jcoop' is a substr of 'jcooper@apl.com')
                for (found = FALSE, ptr = str; *ptr; ptr += len + 1) {
                  // if there's only one token left, then len will = startLen, 
                  // and we'll iterate once only
                  startLen = strlen (ptr);
                  if ((len = strcspn (ptr, " ,\n\t\r")) != startLen) {
                        ptr[len] = '\0';        // replace delim with NULL char

                        while (strchr (" ,\n\t\r", ptr[len + 1]))       // eat white space

                          ptr[len++] = '\0';
                  }
                  if (!strcmp (ptr, mybuf)) {
                        found = TRUE;
                        break;
                  }
                  if (len == startLen)  // last token, we're done

                        break;
                }
                if (!found) {
                  if (strlen (mybuf) + strlen (DialogString) + 2 > MAXDIALOGSTRING - 1) {
                        WVMessageBox (hDlg, IDS_WVCNCM_ADD_ADDRESS, IDS_WVCNCM_LONG_ADDRESS, MB_OK);
                        return FALSE;
                  }
                  if (DialogString[0])  // if this isn't the first one, prepend a comma

                        strcat (DialogString, ", ");
                  strcat (DialogString, mybuf);
                  SetDlgItemText (hDlg, IDD_MAIL_DEST, DialogString);
                }
          }
          break;

        case IDD_MAIL_DEST:
          GetDlgItemText (hDlg, IDD_MAIL_DEST, mybuf, MAXDIALOGSTRING - 1);
          EnableWindow (GetDlgItem (hDlg, IDD_ADD), mybuf[0]);
          return FALSE;                         /* let default handler act on these events */
          break;

        case IDD_ADD:
          // check addresses listed on to-line, add each one to list box 
          // (if not already in the list)
          GetDlgItemText (hDlg, IDD_MAIL_DEST, DialogString, MAXDIALOGSTRING - 1);
          for (ptr = DialogString; *ptr; ptr += len + 1) {
                // if there's only one token left, then len will = startLen, 
                // and we'll iterate once only
                startLen = strlen (ptr);
                if ((len = strcspn (ptr, " ,\n\t\r")) != startLen) {
                  ptr[len] = '\0';              // replace delim with NULL char

                  while (strchr (" ,\n\t\r", ptr[len + 1]))             // eat white space

                        ptr[len++] = '\0';
                }
                if (*ptr != '\0')               // if not a blank
                 {
                  // check if it's already in the list
                  count = (int) SendMessage (hMailList, LB_GETCOUNT, 0, 0L);
                  for (j = 0; j < count; j++) {
                        SendMessage (hMailList, LB_GETTEXT, j, (LPARAM) (LPSTR) mybuf);
                        if (!stricmp (ptr, mybuf))
                          break;
                  }
                  if (j == count)               // if not already in list, add it

                        SendMessage (hMailList, LB_ADDSTRING, 0, (LPARAM) (LPSTR) ptr);
                }
                if (len == startLen)    // last token, we're done

                  break;
          }
          break;

        case IDD_DELETE:
          // remove all selected addresses from list box
          count = (int) SendMessage (hMailList, LB_GETCOUNT, 0, 0L);

          SendMessage (hMailList, WM_SETREDRAW, FALSE, 0L);

          // working backwards, delete selected items
          for (j = count - 1; j >= 0; j--)
                if (SendMessage (hMailList, LB_GETSEL, j, 0L))
                  SendMessage (hMailList, LB_DELETESTRING, j, 0L);

          SendMessage (hMailList, WM_SETREDRAW, TRUE, 0L);
          EnableWindow (GetDlgItem (hDlg, IDD_DELETE), FALSE);
          break;

        case IDOK:
          GetDlgItemText (hDlg, IDD_MAIL_DEST, mybuf, MAXDIALOGSTRING - 1);

          // make sure the address (if more than one) is comma separated
          DialogString[0] = '\0';
          for (ptr = mybuf; *ptr; ptr += len + 1) {
                // if there's only one token left, then len will = startLen, 
                // and we'll iterate once only
                startLen = strlen (ptr);
                if ((len = strcspn (ptr, " ,\n\t\r")) != startLen) {
                  ptr[len] = '\0';              // replace delim with NULL char

                  while (strchr (" ,\n\t\r", ptr[len + 1]))             // eat white space

                        ptr[len++] = '\0';
                }

                strcat (DialogString, ptr);

                if (len == startLen)    // last token, we're done

                  break;
                else
                  strcat (DialogString, ", ");
          }

          // commit changes in listbox to internal MailList
          ResetTextBlock (MailList);
          count = (int) SendMessage (hMailList, LB_GETCOUNT, 0, 0L);
          for (j = 0; j < count; j++) {
                SendMessage (hMailList, LB_GETTEXT, j, (LPARAM) (LPSTR) mybuf);
                AddLineToTextBlock (MailList, mybuf);
          }

          EndDialog (hDlg, TRUE);
          break;


        case IDCANCEL:
          // changes to MailList are _not_ committed on Cancel
          EndDialog (hDlg, FALSE);
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}


/*-- function WinVnSmartFilerDialog ---------------------------------------
 *
 *  Manages smart filer config
 *  jsc 9/10/94 -- and 7500 mile tuneup 9/30/94
 */

BOOL CALLBACK WinVnSmartFilerDlg (HWND hDlg, unsigned int iMessage, WPARAM wParam, LPARAM lParam)
{
  static HWND hMapList;                 /* extension mapping combobox */
  static HWND hDupeList, hShortenList;  /* options dropllists */
  static int MyEnableExtensionConversion;
  register int j;
  int count, n, e, tabStop[10];
  char sBuff1[MAXDIALOGSTRING];
  char sBuff2[MAXDIALOGSTRING];
  char sBuff3[MAXDIALOGSTRING];
  //char mybuf[MAXDIALOGSTRING];
  RECT rect;
  BOOL ok;

  switch (iMessage) {
  case WM_INITDIALOG:
        hMapList = GetDlgItem (hDlg, IDD_EXT_MAP_LIST);
        hDupeList = GetDlgItem (hDlg, IDD_AVOID_DUPES);
        hShortenList = GetDlgItem (hDlg, IDD_NAME_SHORTEN);

        /* set tab stop at half width (width/2) of listbox.
         * we need # of dlg units to get half width.
         * a dlg unit is (baseUnit/4).  so half posn in dlg units is...
         *      (width/2) / (baseUnit/4) ==> (2*cx)/baseUnit
         *
         * then, set a few more evenly spaced after that in case 
         * of really long source extension in list
         */
        GetWindowRect (hMapList, &rect);
        tabStop[0] = (2 * (RectWidth (rect)) / LOWORD (GetDialogBaseUnits ()));
        for (j = 1; j < 10; j++) {
          tabStop[j] = tabStop[j - 1] + 2 * LOWORD (GetDialogBaseUnits ());
        }
        SendMessage (hMapList, LB_SETTABSTOPS, 10, (LPARAM) tabStop);

        SetDlgItemInt (hDlg, IDD_MAX_NAME_LEN, MaxFileNameLen, TRUE);
        SetDlgItemInt (hDlg, IDD_MAX_EXT_LEN, MaxFileExtLen, TRUE);

        CheckDlgButton (hDlg, IDD_ENABLE_CONVERSION, EnableExtensionConversion);
		CheckDlgButton (hDlg, IDC_CB_NOSPACE, SpaceProcessing);
        EnableWindow (hMapList, EnableExtensionConversion);
        EnableWindow (GetDlgItem (hDlg, IDD_SOURCE_EXT), EnableExtensionConversion);
        EnableWindow (GetDlgItem (hDlg, IDD_DOS_EXT), EnableExtensionConversion);

        EnableWindow (GetDlgItem (hDlg, IDD_ADD), FALSE);
        EnableWindow (GetDlgItem (hDlg, IDD_DELETE), FALSE);

        SendMessage (hMapList, WM_SETREDRAW, FALSE, 0L);
        SendMessage (hDupeList, WM_SETREDRAW, FALSE, 0L);
        SendMessage (hShortenList, WM_SETREDRAW, FALSE, 0L);
        for (j = 0; j < (int) ExtMapSourceList->numLines; j++) {
          _snprintf (sBuff1, MAXDIALOGSTRING, "%s\t%s",
          TextBlockLine (ExtMapSourceList, j), TextBlockLine (ExtMapDosList, j));
          SendMessage (hMapList, LB_ADDSTRING, 0, (LPARAM) (LPCSTR) sBuff1);
        }
    LoadString(hInst, IDS_SFPRONEWFIL, sBuff1, sizeof(sBuff1)); // "Prompt for new file name"
        SendMessage (hDupeList, CB_INSERTSTRING, (WPARAM) DUPE_AVOID_NONE, (LPARAM) (LPCSTR) sBuff1);
    LoadString(hInst, IDS_SFREPEXTNUM, sBuff2, sizeof(sBuff2)); // "Replace extension with number"
        SendMessage (hDupeList, CB_INSERTSTRING, (WPARAM) DUPE_AVOID_NUMBER_EXT, (LPARAM) (LPCSTR) sBuff2);
    LoadString(hInst, IDS_SFPRENUMNAM, sBuff2, sizeof(sBuff2)); // "Preserve extension, prepend number to name"
        SendMessage (hDupeList, CB_INSERTSTRING, (WPARAM) DUPE_AVOID_PREPEND_NUM, (LPARAM) (LPCSTR) sBuff2);
    LoadString(hInst, IDS_SFAPPNUMNAM, sBuff2, sizeof(sBuff2)); // "Preserve extension, append number to name"
        SendMessage (hDupeList, CB_INSERTSTRING, (WPARAM) DUPE_AVOID_APPEND_NUM, (LPARAM) (LPCSTR) sBuff2);

        SendMessage (hShortenList, CB_INSERTSTRING, (WPARAM) SHORTEN_NONE, (LPARAM) (LPCSTR) sBuff1);
    LoadString(hInst, IDS_SFTRUMAXLEN, sBuff2, sizeof(sBuff2)); // "Truncate at max name length"
        SendMessage (hShortenList, CB_INSERTSTRING, (WPARAM) SHORTEN_TRUNCATE, (LPARAM) (LPCSTR) sBuff2);
    LoadString(hInst, IDS_SFCONSKPPUN, sBuff2, sizeof(sBuff2)); // "Condense (skip vowels, punctuation)"
        SendMessage (hShortenList, CB_INSERTSTRING, (WPARAM) SHORTEN_SKIP_VOWELS, (LPARAM) (LPCSTR) sBuff2);

        MyEnableExtensionConversion = EnableExtensionConversion;

        SendMessage (hDupeList, CB_SETCURSEL, (WPARAM) OnDupeName, 0L);
        SendMessage (hShortenList, CB_SETCURSEL, (WPARAM) OnNameTooLong, 0L);

        SendMessage (hMapList, WM_SETREDRAW, TRUE, 0L);
        SendMessage (hDupeList, WM_SETREDRAW, TRUE, 0L);
        SendMessage (hShortenList, WM_SETREDRAW, TRUE, 0L);

        return TRUE;
        break;
	case WM_HELP:
		DoHelp (hDlg, HELP_CONTEXT, (DWORD)404); //IDD_WINVNSMARTFILER
		break;


  case WM_COMMAND:
        switch (LOWORD (wParam)) {


        case IDD_EXT_MAP_LIST:
          switch (GET_WM_COMMAND_CMD (wParam, lParam)) {
          case LBN_SELCHANGE:
                EnableWindow (GetDlgItem (hDlg, IDD_DELETE), TRUE);
                return FALSE;                   /* let the default handler do it */
                break;
          }
          break;

        case IDD_SOURCE_EXT:
        case IDD_DOS_EXT:
          GetDlgItemText (hDlg, IDD_SOURCE_EXT, str, sizeof(str) - 1);
      strcpy(sBuff1, ".");
      strcat(sBuff1, ('.' == str[0]) ? &str[1] : &str[0]);
          GetDlgItemText (hDlg, IDD_DOS_EXT, str, sizeof(str) - 1);
      strcpy(sBuff2, ".");
      strcat(sBuff2, ('.' == str[0]) ? &str[1] : &str[0]);
          EnableWindow (GetDlgItem (hDlg, IDD_ADD), (sBuff1[1] || sBuff2[1]));
          return FALSE;                         /* let default handler act on these events */
          break;

        case IDD_ADD:
          GetDlgItemText (hDlg, IDD_SOURCE_EXT, str, sizeof(str) - 1);
      strcpy(sBuff1, ".");
      strcat(sBuff1, ('.' == str[0]) ? &str[1] : &str[0]);
          GetDlgItemText (hDlg, IDD_DOS_EXT, str, sizeof(str) - 1);
      strcpy(sBuff2, ".");
      strcat(sBuff2, ('.' == str[0]) ? &str[1] : &str[0]);
          _snprintf(sBuff3, sizeof(sBuff3), "%s\t%s", sBuff1, sBuff2);

          /* check if it's already in the list */
          count = (int) SendMessage (hMapList, LB_GETCOUNT, 0, 0L);
          for (j = 0; j < count; j++) {
                SendMessage (hMapList, LB_GETTEXT, (WPARAM) j, (LPARAM) (LPSTR) str);
                /* is the source ext already in the list somewhere? */
                if (!strnicmp (sBuff3, str, strlen(sBuff1) + 1))
                  break;
          }
          if (j == count) {                     /* if not already in list, add it */
                SendMessage (hMapList, LB_ADDSTRING, 0, (LPARAM) (LPSTR) sBuff3);
          }
          else {                                        /* if already in list, modify the dos ex */
                SendMessage (hMapList, LB_DELETESTRING, (WPARAM) j, 0L);
                SendMessage (hMapList, LB_INSERTSTRING, (WPARAM) j, (LPARAM) (LPSTR) sBuff3);
          }
          break;

        case IDD_DELETE:
          j = (int) SendMessage (hMapList, LB_GETCURSEL, 0, 0L);
          SendMessage (hMapList, LB_DELETESTRING, (WPARAM) j, 0L);
          EnableWindow (GetDlgItem (hDlg, IDD_DELETE), FALSE);
          break;

        case IDOK:
          n = GetDlgItemInt (hDlg, IDD_MAX_NAME_LEN, &ok, TRUE);
          e = GetDlgItemInt (hDlg, IDD_MAX_EXT_LEN, &ok, TRUE);
#ifdef _WIN32
          if (n + e <= 0 || n + e > 255) {
          WVMessageBox (hDlg, IDS_SFNAMEWRONGLEN1, IDS_SFNAMEERR, MB_OK);
                break;
          }
          if (e == 0 && MyEnableExtensionConversion) {
                if (WVMessageBox (hDlg, IDS_SFNAMEWRONGLEN2, IDS_SFWARNING, MB_YESNO) == IDNO)
                  break;
                else
                  MyEnableExtensionConversion = FALSE;
          }

#else
          if (n <= 0 || n > 8) {
                WVMessageBox (hDlg, IDS_SFDOSNAMLEN, IDS_SFFILNAMLERR, MB_OK);
                break;
          }
          if (e <= 0 || e > 3) {
                WVMessageBox (hDlg, IDS_SFDOSEXTLEN, IDS_SFFILEXTLERR, MB_OK);
                break;
          }
#endif /* _WIN32 */
          MaxFileNameLen = n;
          MaxFileExtLen = e;
          OnDupeName = (int) SendMessage (hDupeList, CB_GETCURSEL, 0, 0L);
          OnNameTooLong = (int) SendMessage (hShortenList, CB_GETCURSEL, 0, 0L);
          EnableExtensionConversion = MyEnableExtensionConversion;
		  SpaceProcessing = (IsDlgButtonChecked (hDlg, IDC_CB_NOSPACE) != 0);
          
		  /* commit changes in listbox to internal lists */
          ResetTextBlock (ExtMapSourceList);
          ResetTextBlock (ExtMapDosList);
          count = (int) SendMessage (hMapList, LB_GETCOUNT, 0, 0L);
          for (j = 0; j < count; j++) {
                SendMessage (hMapList, LB_GETTEXT, j, (LPARAM) (LPSTR) sBuff1);
                sscanf (sBuff1, "%s\t%s", sBuff2, sBuff3);
                AddLineToTextBlock (ExtMapSourceList, sBuff2);
                AddLineToTextBlock (ExtMapDosList, sBuff3);
          }

          EndDialog (hDlg, TRUE);
          break;


        case IDCANCEL:
          /* changes to lists are _not_ committed on Cancel */
          EndDialog (hDlg, FALSE);
          break;

        case IDD_ENABLE_CONVERSION:
          MyEnableExtensionConversion = !MyEnableExtensionConversion;
          CheckDlgButton (hDlg, IDD_ENABLE_CONVERSION, MyEnableExtensionConversion);
          SendMessage (hMapList, LB_SETCURSEL, (WPARAM) - 1, 0L);       /* deselect */
          EnableWindow (hMapList, MyEnableExtensionConversion);
          EnableWindow (GetDlgItem (hDlg, IDD_SOURCE_EXT), MyEnableExtensionConversion);
          EnableWindow (GetDlgItem (hDlg, IDD_DOS_EXT), MyEnableExtensionConversion);
          if (!MyEnableExtensionConversion) {
                EnableWindow (GetDlgItem (hDlg, IDD_ADD), FALSE);
                EnableWindow (GetDlgItem (hDlg, IDD_DELETE), FALSE);
          }
          break;

        default:
          return FALSE;
        }
        break;

  default:
        return FALSE;
        break;
  }
  return TRUE;
}

BOOL CALLBACK WinVnSMTPDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
#define MAXSTR  32


	char SMTPPassword[MAXNNTPSIZE];
	BOOL bOK;

	static HFONT hDlgFont = NULL;
	

	switch (iMessage) 
	{

	case WM_INITDIALOG:
		CenterWindow (hDlg, GetParent(hDlg) );
		/* Get dialog font and create non-bold version */ 
		hDlgFont = NULL;
		if (g_bIsXP && !hDlgFont)
		{  //fix password font
			LOGFONT lFont;
			lFont.lfHeight = 14;
			lFont.lfWidth = 0;
			lFont.lfEscapement = 0;
			lFont.lfOrientation = 0;
			lFont.lfWeight = 200; // non-bold font weight
			lFont.lfItalic = 0;
			lFont.lfUnderline = 0;
			lFont.lfStrikeOut = 0;
			lFont.lfCharSet = ANSI_CHARSET;
			lFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
			lFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
			lFont.lfQuality = DEFAULT_QUALITY;
			lFont.lfPitchAndFamily = VARIABLE_PITCH | FF_SWISS;
			strcpy(lFont.lfFaceName, "Microsoft Sans Serif");
			hDlgFont = CreateFontIndirect(&lFont);

			SendDlgItemMessage(hDlg, ID_CONFIG_AUTH_PASSWORD, WM_SETFONT,  (WPARAM)hDlgFont, (DWORD)TRUE);
			// Send WM_SETFONT message to desired controls

		}

		CheckDlgButton (hDlg, ID_CONFIG_USESSL, UseSSL);

		SetDlgItemText (hDlg, ID_CONFIG_SMTP_SERVER, SMTPHost);
		SetDlgItemText (hDlg, ID_CONFIG_SMTP_PORT, SMTPService);

		SetDlgItemText (hDlg, ID_SMTP_USER, SMTPUserName);
		MRRDecrypt (SMTPPasswordEncrypted, (unsigned char *) SMTPPassword, MAXNNTPSIZE);
		SetDlgItemText (hDlg, ID_SMTP_PASS, SMTPPassword);

		
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_SMTP_SERVER), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_SMTP_PORT), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_CONFIG_SMTP_SERVER), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_SMTP_USER), MAXNNTPSIZE);
		Edit_LimitText(GetDlgItem(hDlg, ID_SMTP_PASS), MAXNNTPSIZE);

#ifdef _INCLSSL
		CheckRadioButton (hDlg, ID_CONFIG_SMTP, ID_CONFIG_SMTPTLS, ID_CONFIG_SMTP+SMTPUseSSL);
#else
		CheckRadioButton (hDlg, ID_CONFIG_SMTP, ID_CONFIG_SMTPTLS, ID_CONFIG_SMTP);
		SendDlgItemMessage (hDlg, ID_CONFIG_SMTPSSL, WM_ENABLE, 0, 0);
		SendDlgItemMessage (hDlg, ID_CONFIG_SMTPTLS, WM_ENABLE, 0, 0);
#endif

		// PCT, 10/05/94
		CheckDlgButton (hDlg, ID_SMTP_SAVEPASS, SMTPSavePassword);

		if (lParam)
		{
			SetFocus( GetDlgItem (hDlg, ID_SMTP_PASS));
			return FALSE;

		}
		return TRUE;
		break;

		case WM_HELP:
			{
				LPHELPINFO lphi = (LPHELPINFO) lParam; 
				
				DoHelp ((HWND)lphi->hItemHandle/*NetDoc.hDocWnd*/, HELP_CONTEXT, (DWORD)IDD_WINVNSMTP);

				if (lphi->iContextType == HELPINFO_WINDOW && lphi->iCtrlId != 9 )
				{
					DWORD dwHelp[4] = {	lphi->iCtrlId, lphi->iCtrlId + 0x10000, 0,0};
					//dwHelp[0] = 401;
					//dwHelp[1] = 401 + 0x10000;
					//WinHelp ((HWND)lphi->hItemHandle/*NetDoc.hDocWnd*/, mybuf, HELP_WM_HELP, (DWORD)(LPVOID)dwHelp);
				}

			}
			break;

		case WM_COMMAND:
		switch (wParam) 
		{
			case IDHELP:
				DoHelp (hDlg, HELP_CONTEXT, (DWORD)IDD_WINVNSMTP);
				break;

			case IDOK:

				SMTPUseSSL = (IsDlgButtonChecked (hDlg, ID_CONFIG_SMTP) ?  0 : 
						IsDlgButtonChecked (hDlg, ID_CONFIG_SMTPSSL) ?  1 :
						IsDlgButtonChecked (hDlg, ID_CONFIG_SMTPTLS) ?  2 : 0);
				
				GetDlgItemText (hDlg, ID_CONFIG_SMTP_SERVER, SMTPHost, MAXNNTPSIZE);
				if (*SMTPHost == '\0') 
				{
					WVMessageBox (hDlg, IDS_WVCNCM_INVALID_MAIL, IDS_WVCNCM_INVALID_SMTP, MB_OK | MB_ICONSTOP);
					break;
				}
				GetDlgItemText (hDlg, ID_CONFIG_SMTP_PORT, SMTPService, MAXNNTPSIZE);
				if (*SMTPService == '\0') 
				{
					WVMessageBox (hDlg, IDS_WVCNCM_ENTER_SMTPPORT, IDS_WVCNCM_INVALID_SMTPPORT, MB_OK | MB_ICONSTOP);
					break;
				}

				// PCT, 10/05/94
				SMTPSavePassword = IsDlgButtonChecked (hDlg, ID_SMTP_SAVEPASS);
				GetDlgItemText (hDlg, ID_SMTP_USER, SMTPUserName, MAXNNTPSIZE);
				GetDlgItemText (hDlg, ID_SMTP_PASS, SMTPPassword, MAXNNTPSIZE);
				if (*SMTPPassword)
						MRREncrypt ((unsigned char *) SMTPPassword, strlen (SMTPPassword) + 1, SMTPPasswordEncrypted);
				else
					*SMTPPasswordEncrypted = '\0';

				EndDialog (hDlg, TRUE);
			break;

			case IDCANCEL:
				EndDialog (hDlg, FALSE);
				break;

			case ID_CONFIG_SMTP:
				SetDlgItemText (hDlg, ID_CONFIG_SMTP_PORT, "smtp");	//25
				break;
			case ID_CONFIG_SMTPSSL:
				SetDlgItemText (hDlg, ID_CONFIG_SMTP_PORT, "465"); //smtps 
				break;
			case ID_CONFIG_SMTPTLS:
				SetDlgItemText (hDlg, ID_CONFIG_SMTP_PORT, "587"); //submission
				break;

			default:
				return FALSE;
		}
		break;

	default:
			return FALSE;
			break;
	}
	return TRUE;
}


